


#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <ctype.h>

#include <time.h>

#include "dataspy.h"
#include "transfer.h"

//    24 byte data block header - used for data blocks in shared memory and data blocks on disc or tape.

typedef struct s_data_header {
    char header_id[8];
    int header_sequence;
    short  header_stream;
    short  header_DataWidth;
    short  header_MyEndian;
    short  header_DataEndian;
    int header_dataLen;
} DATA_HEADER;


int convertFEE64toMBS(char *, int);

void byteswop16(char*, int);
void byteswop32(char*, int);

#include <signal.h>

int MBSBuffer[64*1024];     // maximum length is 256 Kbytes
int MBSBufferLen = 16 * 1024 * 4;      // default length is 64 Kbytes

unsigned long long TimeStampF = 0;
unsigned long long WRTimeF = 0;
unsigned long long MBSTimeF = 0;

unsigned int Time4863 = 0;
unsigned int Time2847 = 0;
unsigned int Time0027 = 0;

unsigned int WR4863 = 0;
unsigned int WR2847 = 0;
unsigned int WR0027 = 0;

int Verbose = 0;

void Usage(char *progname) {
    fprintf(stderr,"Usage\n%s -n server -t [delayticks] -p [port] -m [mode]\n",progname);
    fprintf(stderr,"\tserver is the IP address or name of destination\n");
    fprintf(stderr,"\tdelayticks is the delay (in millisecs) when looking for buffers\n");
    fprintf(stderr,"\tport is the TCP port to use\n");
    fprintf(stderr,"\tmode is the protocol transfer option\n");
    fprintf(stderr,"Defaults are (none); 6500; 4\n");
    exit(1);
}


int main(int argc, char **argv) {

    char * Buffer;
    int BufferState;
    int BufferLength;

int DataLength;
int BufferSize = 64*1024;
DATA_HEADER data_header;

    DATA_HEADER * DataBlock;

    short DataEndian;
    short DataWidth;
    int dataLen;

    char *TS_Server= NULL;
    int Port=6500;
    int NoBlock=0;
    unsigned int stream = 1;
    int s;
    int i, j;

    unsigned short * p16;
    unsigned int * p32;
    unsigned char * p8;

    int DelayTicks=10;

    unsigned int MBS0015;
    unsigned int MBS1631;
    unsigned int MBS3247;
    unsigned int MBS4863;

    unsigned int MBSSubSystemID=0x00000200;

    int mbsbufferlen;  /*  length in ints  */
    int * mbsbufferptr;

    mbsbufferptr = MBSBuffer;
    mbsbufferlen = 0;

    int mbslen = 0;

    struct timespec time_request;


    if (argc >1) {
        for(i=1;i <argc;i++) {
            if ( (argv[i][0] == '-') || (argv[i][0] == '/') ) {
                switch(tolower(argv[i][1])) {
                    case 'n':
                        TS_Server = argv[++i];
                        break;
                    case 't':
                        DelayTicks = atoi(argv[++i]);
                        break;
                    case 'p':
                        Port = atoi(argv[++i]);
                        break;
                    case 'k':
                        NoBlock = atoi(argv[++i]);
                        break;
                    case 's':
                        MBSBufferLen = atoi(argv[++i])*1024;
                        break;
                    case 'v':
                        Verbose = atoi(argv[++i]);
                        break;
                    default:
                        Usage(argv[0]);
                        break;
                }
            }
            else
                Usage(argv[0]);
        }
    }

    if (TS_Server == NULL)  Usage(argv[0]);

//    (void) transferSetVerbose(Verbose > 1);
    (void) transferUseOverlap(0);
    (void) transferBlockMode(NoBlock);
    (void) transferNice(1);
    (void) transferEndian(1);

    (void) transferMode(4);   /*  MBS  */
    (void) transferBlockSize(MBSBufferLen);

    (void) transferPort(Port);
    (void) transferInit(TS_Server);
    printf ("Resetting communication with MBS server\n");

//    send MBS Start Data Token

     mbsbufferptr = MBSBuffer;
     *mbsbufferptr++ = 2;  /* length */
     *mbsbufferptr++ = 0x0001000a;
     *mbsbufferptr++ = 0x2500000e;    /* trigger = 14 */
      mbsbufferlen = 3;

     *mbsbufferptr++ = 0xffffffff;   /* end of block */
     mbsbufferlen ++;

     s = transferTxData ((char *)&MBSBuffer[0] , stream, MBSBufferLen);

     if (Verbose > 1) printf("Done with code=%d\n",s);

    i = dataSpyOpen (0);
    printf ("open = %d\n", i);

    for (;;)
    {

/*    obtain a data buffer from shared memory. This will have a 24 byte header */

       DataLength = dataSpyRead (0, (char*)&Buffer[sizeof(HEADER)-sizeof(DATA_HEADER)], BufferSize);  /* read into buffer at offset 8 bytes; real data starts at 8+24 = 32  */
       if (Verbose) printf("read done with code=%d\n",DataLength);

       if (DataLength > 0) {

              memcpy (data_header.header_id, (char*)&Buffer[sizeof(HEADER)-sizeof(DATA_HEADER)], sizeof(DATA_HEADER));
              BufferLength = data_header.header_dataLen;   /* the actual application data less headers  */
              if (Verbose) printf("buffer length=%d\n",BufferLength);
         
/*    transfer this data buffer.  This requires that we allow 32 bytes for use by the protocol overheads    */

         /*   write data from offset 32 bytes  */
//         s = transferMultiTxData (ID, &Buffer[sizeof(HEADER)], stream, BufferLength);  


//       Buffer = (char*) get_block(&BufferState, &BufferLength);
       DataBlock = (DATA_HEADER*) Buffer;

//       if (BufferState != 0) {

    if (Verbose > 1) {
        printf("\n\nBlock >>>\n");

        p16 = (unsigned short *)Buffer;

        for (j = 0; j <64;) {
           printf(" 0x%04x", p16[j]);
           j++;
           if ((j/16)*16 == j) printf("\n");
        }
        printf("\n");
    }

    if (Verbose > 0) printf(" Block length %d items\n", (dataLen-24)/8);

    DataEndian = DataBlock->header_DataEndian;
    DataWidth = DataBlock->header_DataWidth;
    dataLen = DataBlock->header_dataLen;

    if (Verbose > 1)  printf("Block endian %d: datawidth %d: datalen %d\n",DataEndian,DataWidth,dataLen);

/*
           if (DataBlock->header_DataEndian != 1) {
               switch (DataBlock->header_DataWidth) {
                   case 2: 
                       byteswop16(Buffer, BufferLength);
                       break;
                   case 4: 
                       byteswop32(Buffer, BufferLength);
                       break;
                   case 8: 
//                       byteswop64(Buffer, BufferLength);
                       break;
                   default:
                       byteswop16(Buffer, BufferLength);
                       break;
               }
           }
*/
            
           byteswop32(Buffer, BufferLength);

           if (Verbose > 1) printf("convert %d bytes\n",dataLen);

           mbslen = convertMIDASTDRtoMBS (Buffer+sizeof(DATA_HEADER) , dataLen);

           if (Verbose > 1) printf("convert returned %d ints\n",mbslen);

           if (mbslen > 0) {
              if (Verbose > 1) printf("formatting %d ints\n",mbslen);

              if ((mbslen*4) > MBSBufferLen) {
                  printf("Warning formatting when %d > %d\n",mbslen*4 ,MBSBufferLen);
              }

              if (Verbose > 1) printf ("MBS time = 0x%016llx\n", MBSTimeF);

              mbsbufferptr = &MBSBuffer[0];
              mbsbufferlen = 0;

              *mbsbufferptr++ = 0;  /* length  */
              *mbsbufferptr++ = 0x0001000a;
              *mbsbufferptr++ = 0x25000001;
              *mbsbufferptr++ = MBSSubSystemID;
              mbsbufferlen += 4;

              MBS0015 = (unsigned int) (MBSTimeF & 0x000000000000ffff);
              MBS1631 = (unsigned int) ((MBSTimeF >> 16) & 0x000000000000ffff);
              MBS3247 = (unsigned int) ((MBSTimeF >> 32) & 0x000000000000ffff);
              MBS4863 = (unsigned int) ((MBSTimeF >> 48) & 0x000000000000ffff);

              if (Verbose > 1) printf ("0x%08lx 0x%08lx 0x%08lx 0x%08lx\n", MBS0015, MBS1631, MBS3247, MBS4863);

              *mbsbufferptr++ = 0x03e10000 | MBS0015;
              *mbsbufferptr++ = 0x04e10000 | MBS1631;
              *mbsbufferptr++ = 0x05e10000 | MBS3247;
              *mbsbufferptr++ = 0x06e10000 | MBS4863;
              mbsbufferlen += 4;

              mbsbufferlen += mbslen;      /*  add in data payload  */
              mbsbufferptr += mbslen;

              *mbsbufferptr++ = 0xffffffff;    /*  end of block */
              mbsbufferlen ++;

              MBSBuffer[0] = mbslen + 7;   /*  data length in ints */

              if (Verbose > 1) printf("Calling transferTxData for %d ints\n",mbsbufferlen);
              if (mbsbufferlen*4 > MBSBufferLen) {
                  printf("Warning calling transferTxData when %d > %d\n",mbsbufferlen ,MBSBufferLen);
              }

    if (Verbose > 1) {
        printf("Out >>>\n");

        p32 = (int *)&MBSBuffer[0];

        for (j = 0; j <32;) {
           printf(" 0x%08x", p32[j]);
           j++;
           if ((j/8)*8 == j) printf("\n");
        }
        printf("\n");
    }

              s = transferTxData ((char *)&MBSBuffer[0] , stream, MBSBufferLen);
              if (Verbose > 1) printf("Done with code=%d\n",s);
           }
       } else {
           if (DelayTicks) {
               time_request.tv_sec = DelayTicks/1000;
               time_request.tv_nsec = (DelayTicks - (time_request.tv_sec * 1000)) * 1000000;
               i = nanosleep(&time_request, NULL);
            }
       }
    }
}

void byteswop32(char* p, int l) 
{
    int len;
    char *ps;
    char *pd;
    char b0, b1, b2, b3;

   len = l/4;
   ps = p;
   pd = p;


   while (len > 0)
   {
       b0 = *ps++;
       b1 = *ps++;
       b2 = *ps++;
       b3 = *ps++;

       *pd++ = b3;
       *pd++ = b2;
       *pd++ = b1;
       *pd++ = b0;
   
       len--;
   }
}

void byteswop16(char* p, int l) 
{
    int len;
    char *ps;
    char *pd;
    char b0, b1;

   len = l/2;
   ps = p;
   pd = p;


   while (len > 0)
   {
       b0 = *ps++;
       b1 = *ps++;

       *pd++ = b1;
       *pd++ = b0;
   
       len--;
   }
}



