

puts "loading sys.V1190.tcl"

variable registerlist

lappend registerlist [list NetVar.V1190_NumSamples  "-n V1190_NumSamples -t int"]
lappend registerlist [list NetVar.V1190_VMERetries  "-n V1190_VMERetries -t int"]
lappend registerlist [list NetVar.V1190_BERREnable  "-n V1190_BERREnable -t int"]
lappend registerlist [list NetVar.V1190_VMEBufSize  "-n V1190_VMEBufSize -t int"]

proc Set_V1190_Options {} {puts "Set_V1190_Options"
    variable Options

    load_Options

   if {![info exists Options(V1190_MaxEvtsPerBuf)]}         {set Options(V1190_MaxEvtsPerBuf) 1}
   if {![info exists Options(V1190_MaxBLT)]}                {set Options(V1190_MaxBLT) 1}
   if {![info exists Options(V1190_MemBufSize)]}            {set Options(V1190_MemBufSize) 1024}
   if {![info exists Options(V1190_VMEBufSize)]}            {set Options(V1190_VMEBufSize) 16384}

   if {![info exists Options(V1190_HistEnable)]}           {set Options(V1190_HistEnable) 1}
   if {![info exists Options(V1190_channels)]}             {set Options(V1190_channels) 512}
   if {![info exists Options(V1190_offset)]}               {set Options(V1190_offset) 0}
   if {![info exists Options(V1190_shift)]}                {set Options(V1190_shift) 0}

   if {![info exists Options(V1190_GroupBase)]}            {set Options(V1190_GroupBase) 224}

    update_Options
}

proc V1190_Configure {} {puts "V1190_Configure"
    variable Options

    set Options(V1190_VMEBufSize) [ReadNetVar NetVar.V1190_VMEBufSize]

    update_Options
}

proc Dump_Raw_VME_Data_V1190 {data} {
      
     foreach word $data {
         if {[string range $word 0 1] != "0x"} then {set word 0x$word}
         if {$word == 0xaa5555aa} then {
             append info "$word - end of dump\n"
             break
         }
         if {$word == 0x55aaaa55} then {
             append info "$word - end of event\n"
             continue
         }
         set type [expr $word >> 30 & 3]
         switch $type {
            0 {
             set event_type "data"
             set channel [expr $word >> 21 & 0x1f]
             set ed [expr $word >> 26 & 1]
             set tdc [expr $word  & 0x1fffff]
             append info "[format "0x%8x    %8s   channel= %4i; edge= %3i; time= 0x%6x" $word $event_type $channel $ed $tdc]"
            }
            1 {
             set event_type "header"
             set geo [expr $word & 0x1f]
             set counter [expr $word >> 5 & 0x3fffff]
             append info "[format "0x%8x    %8s   module= %5i; event number= %8i" $word $event_type $geo $counter]"
            }
            2 {
             set event_type "trailer"
             set geo [expr $word & 0x1f]
             set status [expr $word >> 24 & 7]
             set counter [expr $word >> 5 & 0xffff]
             append info "[format "0x%8x    %8s   module= %5i; status=%2i; event data counter= %8i" $word $event_type $geo $status $counter]"
            }
            3 {
             append info "$word - fifo empty"
            }
         }
         append info "\n"
     }
   return $info
}

proc V1190_DAQ-go {} {puts "V1190_DAQ-go"

   set maxmodule 0
   for {set i 1} {$i <= 16} {incr i} {
      set z [catch {ReadVMERegisterAttr v1190#$i.VMEStatus} m]
      if {$z!=0} then {break}
      incr maxmodule
   }

#   GO -  set essential options

       set ::V1190::allmodules 1
               
#    acquisition mode:
#      set trigger matching
       ::V1190::do_cmd_opcode 0x0000

#    readout 
#    disable TDC header & EOB
       ::V1190::do_set_opcode 0x3100 0

   for {set i 1} {$i <= $maxmodule} {incr i} {

      set z [catch {ReadVMERegister v1190#$i.AcqControl} m]
      if {$z == 0} {WriteVMERegister v1190#$i.AcqControl [expr ($m & 0xfb) + 4]}

   }

}

proc V1190_DAQ-halt {} {puts "V1190_DAQ-halt"

   set maxmodule 0
   for {set i 1} {$i <= 16} {incr i} {
      set z [catch {ReadVMERegisterAttr v1190#$i.VMEStatus} m]
      if {$z!=0} then {break}
      incr maxmodule
   }

   for {set i 1} {$i <= $maxmodule} {incr i} {

      set z [catch {ReadVMERegister v1190#$i.AcqControl} m]
      if {$z == 0} {WriteVMERegister v1190#$i.AcqControl [expr ($m & 0xfb)]}

   }
}

proc V1190_DAQ-setup {} {puts "V1190_DAQ-setup"

   set maxmodule 0
   for {set i 1} {$i <= 16} {incr i} {
      set z [catch {ReadVMERegisterAttr v1190#$i.VMEStatus} m]
      if {$z!=0} then {break}
      incr maxmodule
   }

     V1190_Configure

   for {set i 1} {$i <= $maxmodule} {incr i} {
     ::V1190::do_Restore $i      ;#   restore microcontroller parameters
   }

}

proc V1190_DAQ-flush {} {puts "V1190_DAQ-flush"}

proc V1190_DAQ-reset {} {puts "V1190_DAQ-reset"}

proc define_V1190_config {} {puts "define_V1190_config"
    variable Options

   set maxmodule 0
   for {set i 1} {$i <= 16} {incr i} {
      set z [catch {ReadVMERegisterAttr v1190#$i.VMEStatus} m]
      if {$z!=0} then {break}
      incr maxmodule
   }

   if {$maxmodule > 0} then {DefineConfig "1190 0 [format "0x%x" $Options(V1190_GroupBase)]"}

   for {set i 1} {$i <= $maxmodule} {incr i} {

      set z [catch {ReadVMERegisterAttr v1190#$i.Output_Buffer} m]
      if {$z!=0} then {
          puts "Attempting to locate register v1190#$i.Output_Buffer"
          error $m
      }

      DefineConfig "1190 $i [string range [first $m] 2 end]"
   }
}

proc create_V1190_spectra {} {puts "create_V1190_spectra"
    variable Options 

   set maxmodule 0
   for {set i 1} {$i <= 16} {incr i} {
      set z [catch {ReadVMERegisterAttr v1190#$i.VMEStatus} m]
      if {$z!=0} then {break}
      incr maxmodule
   }
   set maxtdc 128

   for {set i 1} {$i <= $maxmodule} {incr i} {

     for {set j 0} {$j <= $maxtdc} {incr j} {

       if {$Options(V1190_HistEnable) > 0} {

#    histograms for the TDC

           set spectrumname v1190#$i.Ch[set j]
           set z [catch {CreateSpectrum1D $spectrumname $Options(V1190_channels)} m]

#    Allow "Spectrum already exists" so that further spectra can be created
           if {$z!=0 && $m==0x30004} then {set z 0}
           if {$z!=0} then {
              puts "Attempting to create spectrum $spectrumname"
              error $m
           } else {
              set group  [expr (($i-1) * $maxtdc) + $Options(V1190_GroupBase)] 
              map_spectrum  "$spectrumname $group $j $Options(V1190_offset) $Options(V1190_shift)"
           }
        }
      }
   }
}

