

puts "loading sys.V1740.tcl"

variable registerlist

lappend registerlist [list NetVar.V1740_NumSamples  "-n V1740_NumSamples -t int"]
lappend registerlist [list NetVar.V1740_VMERetries  "-n V1740_VMERetries -t int"]
lappend registerlist [list NetVar.V1740_BERREnable  "-n V1740_BERREnable -t int"]
lappend registerlist [list NetVar.V1740_VMEBufSize  "-n V1740_VMEBufSize -t int"]

variable V1740_Base 96

proc Set_V1740_Options {} {
    variable Options
    variable V1740_Base

puts "Set_V1740_Options"

    load_Options

   if {![info exists Options(V1740_MaxEvtsPerBuf)]}         {set Options(V1740_MaxEvtsPerBuf) 1}
   if {![info exists Options(V1740_MaxBLT)]}                {set Options(V1740_MaxBLT) 1}
   if {![info exists Options(V1740_MemBufSize)]}            {set Options(V1740_MemBufSize) 1024}
   if {![info exists Options(V1740_VMEBufSize)]}            {set Options(V1740_VMEBufSize) 16384}

   if {![info exists Options(V1740_Hist_Trace.Enable)]}    {set Options(V1740_Hist_Trace.Enable) 1}
   if {![info exists Options(V1740_Hist_Trace.channels)]}  {set Options(V1740_Hist_Trace.channels) 512}
   if {![info exists Options(V1740_Hist_Trace.offset)]}    {set Options(V1740_Hist_Trace.offset) 0}
   if {![info exists Options(V1740_Hist_Trace.shift)]}     {set Options(V1740_Hist_Trace.shift) 0}

   if {![info exists Options(V1740_GroupBase)]}             {set Options(V1740_GroupBase) $V1740_Base}
   set V1740_Base  $Options(V1740_GroupBase)

    update_Options
}

proc V1740_Configure {} {
    variable Options

puts "V1740_Configure"

    set VME_BUFFER_SIZE [ReadNetVar NetVar.V1740_VMEBufSize]
    set Options(V1740_VMEBufSize) $VME_BUFFER_SIZE

      update_Options

           switch $Options(V1740_MemBufSize) {
              65536 {set BUFF_ORG 2; set BUFF_NUM 4}
              32768 {set BUFF_ORG 3; set BUFF_NUM 8}
              16384 {set BUFF_ORG 4; set BUFF_NUM 16}
              8192  {set BUFF_ORG 5; set BUFF_NUM 32}
              4096  {set BUFF_ORG 6; set BUFF_NUM 64}
              2048  {set BUFF_ORG 7; set BUFF_NUM 128}
              1024  {set BUFF_ORG 8; set BUFF_NUM 256}
              512   {set BUFF_ORG 9; set BUFF_NUM 512}
              256   {set BUFF_ORG 10; set BUFF_NUM 1024}
              default {set BUFF_ORG 10; set BUFF_NUM 1024}
           }

      set z [catch {ReadVMERegisterAttr v1740#1.VMEStatus} m]
      if {$z==0} then {

           WriteVMERegister v1740#1.VMEControl 16   ;# force BERR option

           set SIZE [ReadVMERegister v1740#1.CustomSize]
           if {$SIZE == 0} {set SIZE 700}
           WriteNetVar NetVar.V1740_NumSamples [expr ($SIZE * 3) / 2]  ;# convert to samples

           set WORDS [expr ($SIZE * 3) / 4]   ;# convert to 32 bit words

           set NUM [expr (($Options(V1740_MemBufSize) - 2) * 2) / $WORDS]
           if {$NUM > $Options(V1740_MaxEvtsPerBuf)} {set NUM $Options(V1740_MaxEvtsPerBuf)}

           set BERR [ReadVMERegister v1740#1.VMEControl]  ;#  enable BERR to finish a VME block transfer
           WriteNetVar NetVar.V1740_BERREnable [expr ($BERR >> 4) & 1]

puts "V1740 Output buffers $BUFF_NUM  @ $Options(V1740_MemBufSize) 32 bit words"
puts "Waveform [expr ($SIZE * 3) / 2] samples"
puts "8 channels; $NUM events/buffer"

           set osize [expr (($WORDS + 2) * $NUM) + 2]
puts "V1740 buffer size used $osize words out of max $Options(V1740_MemBufSize)"

           if {$osize > $Options(V1740_MemBufSize)} {
              puts "Number of samples or Events per buffer is too big\n\
                 $osize > $Options(V1740_MemBufSize) - this is not permitted\n\
                 correct options and try again"
              return
           }

           set BLT [expr $VME_BUFFER_SIZE / (($osize * 8) + 4)]
           if {$BLT > $Options(V1740_MaxBLT)} {set BLT $Options(V1740_MaxBLT)}

puts "BLT Event Number = $BLT"
           set isize [expr (($osize * 8) + 4) * $BLT]
puts "Data Acq Input buffer size used $isize words out of max $VME_BUFFER_SIZE"

puts "V1740_Configure $BERR $SIZE $BUFF_ORG $NUM"

           WriteVMERegisters v1740#*.VMEControl $BERR     ;# set all modules to same setting
           WriteVMERegisters v1740#*.CustomSize $SIZE     ;# set all modules to same setting
           WriteVMERegisters v1740#*.BufferOrg $BUFF_ORG
           WriteVMERegisters v1740#\[?,??\].EvtsPerBuf $NUM
           WriteVMERegisters v1740#*.BLTEventNum $BLT     ;# set all modules to same setting
      }

# puts "V1740_Configure $Options(Use1495)"

#     if {$Options(Use1495) == 1} {WriteVMERegisters v1495#1.ArmLimit $BLT}
#     WriteNetVar NetVar.V1495_UseCtrl  $Options(Use1495)

puts "V1740_Configure finished"

}


proc V1740_DAQ-go {} {

   set maxmodule 0
   for {set i 1} {$i <= 16} {incr i} {
      set z [catch {ReadVMERegisterAttr v1740#$i.VMEStatus} m]
      if {$z!=0} then {break}
      incr maxmodule
   }

   for {set i 1} {$i <= $maxmodule} {incr i} {

      set z [catch {ReadVMERegister v1740#$i.AcqControl} m]
      if {$z == 0} {WriteVMERegister v1740#$i.AcqControl [expr ($m & 0xfb) + 4]}

   }
}

proc V1740_DAQ-halt {} {

   set maxmodule 0
   for {set i 1} {$i <= 16} {incr i} {
      set z [catch {ReadVMERegisterAttr v1740#$i.VMEStatus} m]
      if {$z!=0} then {break}
      incr maxmodule
   }

   for {set i 1} {$i <= $maxmodule} {incr i} {

      set z [catch {ReadVMERegister v1740#$i.AcqControl} m]
      if {$z == 0} {WriteVMERegister v1740#$i.AcqControl [expr ($m & 0xfb)]}

   }
}

proc V1740_DAQ-setup {} {
    variable Options

     V1740_Configure
}

proc V1740_DAQ-flush {} {}

proc V1740_DAQ-reset {} {}

proc define_V1740_config {} {
    variable Options
    variable V1740_Base

#   set maxmodule [inform vmecontrol inquire v1740count]
#   if {[string length $maxmodule] > 2} {set maxmodule 0}

puts "define_V1740_config"

   set maxmodule 0
   for {set i 1} {$i <= 16} {incr i} {
      set z [catch {ReadVMERegisterAttr v1740#$i.VMEStatus} m]
      if {$z!=0} then {break}
      incr maxmodule
   }

   if {$maxmodule > 0} then {DefineConfig "17400 0 [format "0x%x" $V1740_Base]"}

   for {set i 1} {$i <= $maxmodule} {incr i} {

      set z [catch {ReadVMERegisterAttr v1740#$i.Output_Buffer} m]
      if {$z!=0} then {
          puts "Attempting to locate register v1740#$i.Output_Buffer"
          error $m
      }

      DefineConfig "17400 $i [string range [first $m] 2 end]"
   }
}

proc create_V1740_spectra {} {
    variable V1740_Base
    variable Options 

puts "create_V1740_spectra"

   set maxmodule 0
   for {set i 1} {$i <= 16} {incr i} {
      set z [catch {ReadVMERegisterAttr v1740#$i.VMEStatus} m]
      if {$z!=0} then {break}
      incr maxmodule
   }

   for {set i 1} {$i <= $maxmodule} {incr i} {

     if {$Options(V1740_Hist_Trace.Enable) == 1} {

        for {set j 0} {$j < 64} {incr j} {   

#    histograms for the data trace

           set spectrumname v1740#$i.Ch[set j]
           set z [catch {CreateSpectrum1D $spectrumname $Options(V1740_Hist_Trace.channels)} m]

#    Allow "Spectrum already exists" so that further spectra can be created
           if {$z!=0 && $m==0x30004} then {set z 0}
           if {$z!=0} then {
              puts "Attempting to create spectrum $spectrumname"
              error $m
           } else {
              set group  [expr (($i-1) * 64) + $j + 256  + $V1740_Base] 
              map_spectrum  "$spectrumname $group 0 $Options(V1740_Hist_Trace.offset) $Options(V1740_Hist_Trace.shift)"
           }
        }
     }
   }
}

