

puts "loading sys.V1742.tcl"

variable registerlist

lappend registerlist [list NetVar.V1742_NumSamples  "-n V1742_NumSamples -t int"]
lappend registerlist [list NetVar.V1742_VMERetries  "-n V1742_VMERetries -t int"]
lappend registerlist [list NetVar.V1742_BERREnable  "-n V1742_BERREnable -t int"]
lappend registerlist [list NetVar.V1742_VMEBufSize  "-n V1742_VMEBufSize -t int"]

variable V1742_Base 96

proc Set_V1742_Options {} {
    variable Options
    variable V1742_Base

puts "Set_V1742_Options"

    load_Options

   if {![info exists Options(V1742_MaxEvtsPerBuf)]}         {set Options(V1742_MaxEvtsPerBuf) 1}
   if {![info exists Options(V1742_MaxBLT)]}                {set Options(V1742_MaxBLT) 1}
   if {![info exists Options(V1742_MemBufSize)]}            {set Options(V1742_MemBufSize) 1024}
   if {![info exists Options(V1742_VMEBufSize)]}            {set Options(V1742_VMEBufSize) 16384}

   if {![info exists Options(V1742_Hist_Trace.Enable)]}    {set Options(V1742_Hist_Trace.Enable) 1}
   if {![info exists Options(V1742_Hist_Trace.channels)]}  {set Options(V1742_Hist_Trace.channels) 512}
   if {![info exists Options(V1742_Hist_Trace.offset)]}    {set Options(V1742_Hist_Trace.offset) 0}
   if {![info exists Options(V1742_Hist_Trace.shift)]}     {set Options(V1742_Hist_Trace.shift) 0}

   if {![info exists Options(V1742_GroupBase)]}             {set Options(V1742_GroupBase) $V1742_Base}
   set V1742_Base  $Options(V1742_GroupBase)

   set Options(V1742_DPP) 0

    update_Options
}

proc V1742_Configure {} {
    variable Options

puts "V1742_Configure"

    set VME_BUFFER_SIZE [ReadNetVar NetVar.V1742_VMEBufSize]
    set Options(V1742_VMEBufSize) $VME_BUFFER_SIZE

      update_Options

#    WriteNetVar NetVar.V1742_Trig $Options(V1742_Trig)
#    WriteNetVar NetVar.V1742_DVP  $Options(V1742_DVP)
#    WriteNetVar NetVar.V1742_Diag  $Options(V1742_Diag)

           switch $Options(V1742_MemBufSize) {
              65536 {set BUFF_ORG 2; set BUFF_NUM 4}
              32768 {set BUFF_ORG 3; set BUFF_NUM 8}
              16384 {set BUFF_ORG 4; set BUFF_NUM 16}
              8192  {set BUFF_ORG 5; set BUFF_NUM 32}
              4096  {set BUFF_ORG 6; set BUFF_NUM 64}
              2048  {set BUFF_ORG 7; set BUFF_NUM 128}
              1024  {set BUFF_ORG 8; set BUFF_NUM 256}
              512   {set BUFF_ORG 9; set BUFF_NUM 512}
              256   {set BUFF_ORG 10; set BUFF_NUM 1024}
              default {set BUFF_ORG 10; set BUFF_NUM 1024}
           }

      set z [catch {ReadVMERegisterAttr V1742#1.VMEStatus} m]
      if {$z==0} then {

           WriteVMERegister V1742#1.VMEControl 16   ;# force BERR option

           set SIZE [ReadVMERegister V1742#1.CustomSize]
           if {$SIZE == 0} {set SIZE 700}
           WriteNetVar NetVar.V1742_NumSamples [expr ($SIZE * 3) / 2]  ;# convert to samples

           set WORDS [expr ($SIZE * 3) / 4]   ;# convert to 32 bit words

           set NUM [expr (($Options(V1742_MemBufSize) - 2) * 2) / $WORDS]
           if {$NUM > $Options(V1742_MaxEvtsPerBuf)} {set NUM $Options(V1742_MaxEvtsPerBuf)}

           set BERR [ReadVMERegister V1742#1.VMEControl]  ;#  enable BERR to finish a VME block transfer
           WriteNetVar NetVar.V1742_BERREnable [expr ($BERR >> 4) & 1]

puts "V1742 Output buffers $BUFF_NUM  @ $Options(V1742_MemBufSize) 32 bit words"
puts "Waveform [expr ($SIZE * 3) / 2] samples"
puts "8 channels; $NUM events/buffer"

           set osize [expr (($WORDS + 2) * $NUM) + 2]
puts "V1742 buffer size used $osize words out of max $Options(V1742_MemBufSize)"

           if {$osize > $Options(V1742_MemBufSize)} {
              puts "Number of samples or Events per buffer is too big\n\
                 $osize > $Options(V1742_MemBufSize) - this is not permitted\n\
                 correct options and try again"
              return
           }

           set BLT [expr $VME_BUFFER_SIZE / (($osize * 8) + 4)]
           if {$BLT > $Options(V1742_MaxBLT)} {set BLT $Options(V1742_MaxBLT)}

puts "BLT Event Number = $BLT"
           set isize [expr (($osize * 8) + 4) * $BLT]
puts "Data Acq Input buffer size used $isize words out of max $VME_BUFFER_SIZE"

puts "V1742_Configure $BERR $SIZE $BUFF_ORG $NUM"

           WriteVMERegisters V1742#*.VMEControl $BERR     ;# set all modules to same setting
           WriteVMERegisters V1742#*.CustomSize $SIZE     ;# set all modules to same setting
           WriteVMERegisters V1742#*.BufferOrg $BUFF_ORG
           WriteVMERegisters V1742#\[?,??\].EvtsPerBuf $NUM
           WriteVMERegisters V1742#*.BLTEventNum $BLT     ;# set all modules to same setting
      }

puts "V1742_Configure $Options(Use1495)"

    if {$Options(Use1495) == 1} {WriteVMERegisters v1495#1.ArmLimit $BLT}
    WriteNetVar NetVar.V1495_UseCtrl  $Options(Use1495)

puts "V1742_Configure finished"

}

proc V1742_DAQ-go {} {

   set maxmodule 0
   for {set i 1} {$i <= 16} {incr i} {
      set z [catch {ReadVMERegisterAttr v1742#$i.VMEStatus} m]
      if {$z!=0} then {break}
      incr maxmodule
   }

   for {set i 1} {$i <= $maxmodule} {incr i} {

      set z [catch {ReadVMERegister v1742#$i.AcqControl} m]
      if {$z == 0} {WriteVMERegister v1742#$i.AcqControl [expr ($m & 0xfb) + 4]}

   }
}

proc V1742_DAQ-halt {} {

   set maxmodule 0
   for {set i 1} {$i <= 16} {incr i} {
      set z [catch {ReadVMERegisterAttr v1742#$i.VMEStatus} m]
      if {$z!=0} then {break}
      incr maxmodule
   }

   for {set i 1} {$i <= $maxmodule} {incr i} {

      set z [catch {ReadVMERegister v1742#$i.AcqControl} m]
      if {$z == 0} {WriteVMERegister v1742#$i.AcqControl [expr ($m & 0xfb)]}

   }
}

proc V1742_DAQ-setup {} {
    variable Options

     V1742_Configure
}

proc V1742_DAQ-flush {} {}

proc V1742_DAQ-reset {} {}

proc Dump_Raw_V1742_Data {} {

   set maxmodule 0
   for {set i 1} {$i <= 16} {incr i} {
      set z [catch {ReadVMERegisterAttr V1742#$i.VMEStatus} m]
      if {$z!=0} then {break}
      incr maxmodule
   }

    set info "Raw data dump for CAEN V1742\n"

    for {set i 1} {$i <= $maxmodule} {incr i} {

       append info "V1742#$i\n"

        DataAcqCommand 19 "Getting Raw Data" "1 $i" 2
        set data [ReadNetVar NetVar.EXEC.Response]

        foreach word $data {
           if {[string range $word 0 1] != "0x"} then {set word 0x$word}
           if {$word == 0xaa5555aa} then {
               append info "$word - end of dump\n"
               break
           }
           if {$word == 0x55aaaa55} then {
               append info "$word - end of event\n"
               continue
           }
           append info "$word "
        }

        append info "\n"
     }

   return $info
}

proc define_V1742_config {} {
    variable Options
    variable V1742_Base

#   set maxmodule [inform vmecontrol inquire V1742count]
#   if {[string length $maxmodule] > 2} {set maxmodule 0}

puts "define_V1742_config"

   set maxmodule 0
   for {set i 1} {$i <= 16} {incr i} {
      set z [catch {ReadVMERegisterAttr V1742#$i.VMEStatus} m]
      if {$z!=0} then {break}
      incr maxmodule
   }

   if {$maxmodule > 0} then {DefineConfig "1740 0 [format "0x%x" $V1742_Base]"}

   for {set i 1} {$i <= $maxmodule} {incr i} {

      set z [catch {ReadVMERegisterAttr V1742#$i.Output_Buffer} m]
      if {$z!=0} then {
          puts "Attempting to locate register V1742#$i.Output_Buffer"
          error $m
      }

      DefineConfig "1740 $i [string range [first $m] 2 end]"
   }
}

proc create_V1742_spectra {} {
    variable V1742_Base
    variable Options 

puts "create_V1742_spectra"

   set maxmodule 0
   for {set i 1} {$i <= 16} {incr i} {
      set z [catch {ReadVMERegisterAttr V1742#$i.VMEStatus} m]
      if {$z!=0} then {break}
      incr maxmodule
   }

   set UseGroup [ReadNetVar NetVar.UseGroup]

   for {set i 1} {$i <= $maxmodule} {incr i} {

     if {$Options(V1742_Hist_Trace.Enable) == 1} {

        for {set j 0} {$j < 64} {incr j} {   

#    histograms for the data trace

           set spectrumname V1742#$i.Ch[set j]
           set z [catch {CreateSpectrum1D $spectrumname $Options(V1742_Hist_Trace.channels)} m]

#    Allow "Spectrum already exists" so that further spectra can be created
           if {$z!=0 && $m==0x30004} then {set z 0}
           if {$z!=0} then {
              puts "Attempting to create spectrum $spectrumname"
              error $m
           } else {
              set group  [expr (($i-1) * 64) + $j + 256  + $V1742_Base] 
              map_spectrum  "$spectrumname $group 0 $Options(V1742_Hist_Trace.offset) $Options(V1742_Hist_Trace.shift)"
           }
        }
     }
   }
}

