

#    experiment run control for LyrTech - taken from sys.tcl

global MIDAS_DataAcquisitionControlServer_Verbose
set MIDAS_DataAcquisitionControlServer_Verbose 0

global env

global Options

   global tcl_platform
   global TempDir
   switch $tcl_platform(platform) {
      windows {set TempDir [file join c:/ temp tcl[pid]]}
      default {set TempDir [file join /tmp tcl[pid]]}
   }

variable SpecBase [file join $env(MIDASBASE) SpecData LyrTech]
variable SpecPath

proc Set_Options {} {
   global Options

   LoadOptions

   if {![info exists Options(Include.VHSADC)]} {set Options(Include.VHSADC) 1}
   if {![info exists Options(VHSADC.settings)]} {set Options(VHSADC.settings) 25032015}

   if {![info exists Options(VHSADC.MaxADC)]} {set Options(VHSADC.MaxADC) 16}
   if {![info exists Options(VHSADC.MaxModule)]} {set Options(VHSADC.MaxModule) 3}

   if {![info exists Options(VHSADC_GroupBase)]} {set Options(VHSADC_GroupBase) 1}

   if {![info exists Options(Rate.channels)]} {set Options(Rate.channels) 150}
   if {![info exists Options(Stat.channels)]} {set Options(Stat.channels) 150}
   if {![info exists Options(Stat.shift)]} {set Options(Stat.shift) 6}
   if {![info exists Options(Stat.offset)]} {set Options(Stat.offset) 64}

   if {![info exists Options(VHSADC.channels)]} {set Options(VHSADC.channels) 32768}
   if {![info exists Options(VHSADC.offset)]} {set Options(VHSADC.offset) 0}
   if {![info exists Options(VHSADC.shift)]} {set Options(VHSADC.shift) 1}

   if {![info exists Options(VHSADC_Hist_Trace_Enable)]} {set Options(VHSADC_Hist_Trace_Enable) 0}
   if {![info exists Options(VHSADC_Hist_Trace_channels)]} {set Options(VHSADC_Hist_Trace_channels) 1024}
   if {![info exists Options(VHSADC_Hist_Energy_Enable)]} {set Options(VHSADC_Hist_Energy_Enable) 1}
   if {![info exists Options(VHSADC_Hist_Energy_channels)]} {set Options(VHSADC_Hist_Energy_channels) 65536}
   if {![info exists Options(VHSADC_Hist_Energy_shift)]} {set Options(VHSADC_Hist_Energy_shift) 0}
   if {![info exists Options(VHSADC_Hist_BaseLine_Enable)]} {set Options(VHSADC_Hist_BaseLine_Enable) 1}
   if {![info exists Options(VHSADC_Hist_BaseLine_channels)]} {set Options(VHSADC_Hist_BaseLine_channels) 65536}
   if {![info exists Options(VHSADC_Hist_BaseLine_shift)]} {set Options(VHSADC_Hist_BaseLine_shift) 0}
   if {![info exists Options(VHSADC_Hist_Time_Enable)]} {set Options(VHSADC_Hist_Time_Enable) 0}
   if {![info exists Options(VHSADC_Hist_RAW_Enable)]} {set Options(VHSADC_Hist_RAW_Enable) 0}
   if {![info exists Options(VHSADC_Hist_RAW_channels)]} {set Options(VHSADC_Hist_RAW_channels) 2048}
   if {![info exists Options(VHSADC_Hist_Veto_Enable)]} {set Options(VHSADC_Hist_Veto_Enable) 0}
   if {![info exists Options(VHSADC_Hist_Veto_channels)]} {set Options(VHSADC_Hist_Veto_channels) 65536}
   if {![info exists Options(VHSADC_Hist_Fail_Enable)]} {set Options(VHSADC_Hist_Fail_Enable) 0}
   if {![info exists Options(VHSADC_Hist_Fail_channels)]} {set Options(VHSADC_Hist_Fail_channels) 65536}

   SaveOptions

}

proc Dump_Raw_Data {} {

   set info "data $::DataAcquisition::ACQSERVER\n"

   set maxmodule 3

   set info "$maxmodule VHSADC modules in use\n\n"

   for {set i 1} {$i <= $maxmodule} {incr i} {

      append info "module $i\n"

      execute-command 19 "Getting Raw Data" "14 $i" 2
      set data [read-register NetVar.EXEC.Response]

     foreach word $data {
         set word $word
         if {$word == 0xaa5555aa} then {
             append info "\n$word - end of dump\n"
             break
         }
         if {$word == 0x55aaaa55} then {
             append info "\n"
             continue
         }
         append info "$word "
     }
   }

   return "$info"
}

proc create_VHSADC_spectra {} {
   global Options

   set maxadc $Options(VHSADC.MaxADC)
   set maxmodule $Options(VHSADC.MaxModule)

   set Me $::DataAcquisition::ACQSERVER
   puts "Creating spectra for $Me"

#   for the data channels

   for {set i 1} {$i <= $maxmodule} {incr i} {

     if {$Options(VHSADC_Hist_Energy_Enable) == 1} {

        for {set j 1} {$j <= $maxadc} {incr j} {   ;#    histograms for the computed energy

           set spectrumname vhsadc#$i.E$j
           puts "Creating $spectrumname"

           set z [catch {CreateSpectrum1D $spectrumname $Options(VHSADC_Hist_Energy_channels)} m]

#    Allow "Spectrum already exists" so that further spectra can be created
            if {$z!=0 && $m==0x30004} then {set z 0}
            if {$z!=0} then {
              puts "Error attempting to create spectrum $spectrumname"
              error $m
            }

            set z [catch {SpectrumPutTitle $spectrumname [set Me]_$spectrumname} m]
        }
     }

     if {$Options(VHSADC_Hist_BaseLine_Enable) == 1} {

        for {set j 1} {$j <= $maxadc} {incr j} {   ;#    histograms for the base line

           set spectrumname vhsadc#$i.B$j
           puts "Creating $spectrumname"

           set z [catch {CreateSpectrum1D $spectrumname $Options(VHSADC_Hist_BaseLine_channels)} m]

#    Allow "Spectrum already exists" so that further spectra can be created
            if {$z!=0 && $m==0x30004} then {set z 0}
            if {$z!=0} then {
              puts "Error attempting to create spectrum $spectrumname"
              error $m
            }

            set z [catch {SpectrumPutTitle $spectrumname [set Me]_$spectrumname} m]
        }
     }

     if {$Options(VHSADC_Hist_Trace_Enable) == 1} {

        for {set j 1} {$j <= $maxadc} {incr j} {    ;#  normal histogram to show trace buffer

           set spectrumname vhsadc#$i.ch$j
           puts "Creating $spectrumname"

           set z [catch {CreateSpectrum1D $spectrumname $Options(VHSADC_Hist_Trace_channels)} m]

#    Allow "Spectrum already exists" so that further spectra can be created
           if {$z!=0 && $m==0x30004} then {set z 0}
           if {$z!=0} then {
              display-eg-error-report self $m "Attempting to create spectrum $spectrumname"
              clear-footer
              error $m
           }

           set z [catch {SpectrumPutTitle $spectrumname [set Me]_$spectrumname} m]
        }
     }

   }   ;# end of  $i <= $maxmodule
}

proc map_VHSADC_spectra {} {
   global env
   global TempDir
   global Options
   upvar #0 SpectraHandle f

   set Me $::DataAcquisition::ACQSERVER

   catch {file mkdir $TempDir}

      set SpectraDefs [file join $TempDir VHSADCSpectraDefs.dat]

      set z [catch {open $SpectraDefs w} f]
      if {$z != 0} then {
          puts "Cannot open file $SpectraDefs\n$f"
          return
      }

      map_spectrum "Stat 0 0 0 $Options(Stat.shift)" 

#   for the data channels

   set maxadc $Options(VHSADC.MaxADC)
   set maxmodule $Options(VHSADC.MaxModule)

   for {set i 1} {$i <= $maxmodule} {incr i} {

     if {$Options(VHSADC_Hist_Energy_Enable) == 1} {  ;#    energy

        for {set j 1} {$j <= $maxadc} {incr j} {

           set spectrumname vhsadc#$i.E$j
           puts "Mapping $spectrumname"

           set group [expr $Options(VHSADC_GroupBase) + (($i-1)*16) + ($j-1)]
           set item 0
           map_spectrum "$spectrumname $group $item 0 $Options(VHSADC_Hist_Energy_shift)"
       }
     }

     if {$Options(VHSADC_Hist_BaseLine_Enable) == 1} {   ;#  base line

        for {set j 1} {$j <= $maxadc} {incr j} {

           set spectrumname vhsadc#$i.B$j
           puts "Mapping $spectrumname"

           set group [expr $Options(VHSADC_GroupBase) + (($i-1)*16) + ($j-1)]
           set item 1
           map_spectrum "$spectrumname $group $item 0 $Options(VHSADC_Hist_BaseLine_shift)"
       }
     }

     if {$Options(VHSADC_Hist_Trace_Enable) == 1} {

        for {set j 1} {$j <= $maxadc} {incr j} {

           set spectrumname vhsadc#$i.ch$j
           puts "Mapping $spectrumname"

           set group [expr 256 + $Options(VHSADC_GroupBase) + (($i-1)*16) + ($j-1)]
           set item 0
           map_spectrum "$spectrumname $group $item 0 0"
       }
     }
   }

   close $f
}

proc doStop {} {

puts "executing doStop for LyrTech"

     Set_Options; make-UIregisters

     Stop_Electronics

     DataAcqCommand 1 "Halt"

     if {[ReadNetVar NetVar.DACQ.Ctxenable] == 1}  {
         DataAcqCommand 1 "disconnect xfer stream #0" 0 2
     }

     if {[ReadNetVar NetVar.DACQ.Ctx2enable] == 1}  {
         DataAcqCommand 2 "disconnect xfer stream #1" 0 2
     }
}


proc doGo {} {

puts "executing doGo for LyrTech"

     Set_Options; make-UIregisters

     if {[ReadNetVar NetVar.DACQ.Ctxenable] == 1}  {
         DataAcqCommand 3 "connect xfer stream #0" 0 2
     }

     if {[ReadNetVar NetVar.DACQ.Ctx2enable] == 1}  {
         DataAcqCommand 6 "connect xfer stream #1" 0 2
     }

     Reset_Electronics

     DataAcqCommand 2 "Go"

     Go_Electronics 
}

proc doSetUp {} {
    global Options

puts "executing doSetUp for LyrTech"

     Set_Options; make-UIregisters

     DataAcqCommand 1 "Halt"

     unmap_spectra

     delete_spectra

      set Me $::DataAcquisition::ACQSERVER

      CreateSpectrum1D Stat $Options(Stat.channels)
      set z [catch {SpectrumPutTitle Stat [set Me]_Stat} m]

      CreateSpectrum1D Rate $Options(Rate.channels)
      set z [catch {SpectrumPutTitle Rate [set Me]_Rate} m]

    create_VHSADC_spectra

    map_VHSADC_spectra 

    map_spectra

    Setup_Electronics

    ParameterRestore $Options(VHSADC.settings)

    DataAcqCommand 3 "Setup"

    WriteNetVar NetVar.DACQ.PollTicks 100
    WriteNetVar NetVar.DACQ.Crdoutenable 1   ;# enable data readout

    puts "completed doSetUp"

}

   proc ParameterRestore {vl} {
      global TempDir

       set fname [file join $TempDir RestoreState.log]
       set z [catch {open $fname w} fid]
       if {$z != 0} {puts "Settings Restore logfile not created\n$fid\n"}

        puts "Restoring Settings"
        puts $fid "Restoring Settings at [clock format [clock seconds] -format "%B %d, %Y %H:%M:%S"]"

      set ::IDS::DBNodeBase $vl
      set ::IDS::DBNodeName $::DataAcquisition::ACQSERVER
      
      puts $fid "Restoring VHSADC settings from $::IDS::DBRootName/$::IDS::DBNodeBase for $::DataAcquisition::ACQSERVER"
      puts "Restoring VHSADC settings from $::IDS::DBRootName/$::IDS::DBNodeBase for $::DataAcquisition::ACQSERVER"

      puts "VHSADC settings restored"
      puts $fid "VHSADC settings restored from $::IDS::DBRootName/$::IDS::DBNodeBase for $::DataAcquisition::ACQSERVER"

#       ::IDS::WriteRegister HasBeenRestored 1

#        Elog INFORMATION "FEE64 settings restored from $::IDS::DBRootName/$::IDS::DBNodeBase for $::DataAcquisition::ACQSERVER"

     close $fid

   }

   proc ParameterSave {vl} {
      global TempDir

       set fname [file join $TempDir SaveState.log]
       set z [catch {open $fname w} fid]
       if {$z != 0} {puts "Settings Save logfile not created\n$fid\n"}

        puts "Saving Settings"
        puts $fid "Saving Settings at [clock format [clock seconds] -format "%B %d, %Y %H:%M:%S"]"

      set ::IDS::DBNodeBase $vl
      set ::IDS::DBNodeName $::DataAcquisition::ACQSERVER
      
      puts $fid "Saving VHSADC settings to $::IDS::DBRootName/$::IDS::DBNodeBase for $::DataAcquisition::ACQSERVER"


      puts $fid "VHSADC settings saved to $::IDS::DBRootName/$::IDS::DBNodeBase for $::DataAcquisition::ACQSERVER"

#       ::IDS::WriteRegister HasBeenRestored 1

#        Elog INFORMATION "FEE64 settings saved to $::IDS::DBRootName/$::IDS::DBNodeBase for $::DataAcquisition::ACQSERVER"

        puts  "Settings saved"
        puts $fid "Settings saved at [clock format [clock seconds] -format "%B %d, %Y %H:%M:%S"]"

        close $fid

   }


proc Reset_Electronics {} {    ;#   used by RESET

}

proc Setup_Electronics {} {    ;#   used by SETUP

}

proc Stop_Electronics {} {

#    inform vhsadc do_acqGO 0

}

proc Go_Electronics {} {

#    inform vhsadc do_acqGO 1

}

 proc SpecSave1D {spec base range type title counts} {
    global tcl_platform
    variable SpecPath

      set f3 [file join $SpecPath $spec.msf]
      set z1 [catch {open $f3 w} m1]
      if {$z1 != 0} then {puts "Cannot create $f3 - $m1"}

      if {$tcl_platform(platform) != "windows"} {
           set z2 [catch {file attributes $f3 -permissions rwxrwxrwx} m2]
           if {$z2 != 0} then {puts "error accessing $f3 - $m2"}
      }

      set f $m1

      fconfigure $f -translation binary -encoding binary

  set ctime [clock format [clock seconds] -format {%d-%b-%Y %H:%M:%S}]
  set mtime [clock format [clock seconds] -format {%d-%b-%Y %H:%M:%S}]

      puts -nonewline $f [binary format i1 412900921]   ;#   magic number
      puts -nonewline $f [binary format i1 1]           ;#   version
      puts -nonewline $f [binary format a32 $spec]      ;#   spectrum name
      puts -nonewline $f [binary format i1 1]           ;#   dimension
      puts -nonewline $f [binary format a20 $ctime]     ;#   creation time/date
      puts -nonewline $f [binary format a20 $mtime]     ;#   modification time/date
      puts -nonewline $f [binary format i1 $base]       ;#   base0

      for {set i 2} {$i <=8} {incr i} {
         puts -nonewline $f [binary format i1 -1]     ;#   baseI
      }

      puts -nonewline $f [binary format i1 $range]    ;#   range0

      for {set i 2} {$i <=8} {incr i} {
         puts -nonewline $f [binary format i1 -1]     ;#   rangeI
      }

      puts -nonewline $f [binary format i1 -1]        ;#   spectrum title offset

      for {set i 2} {$i <=32} {incr i} {
         puts -nonewline $f [binary format i1 -1]     ;#  information pointers
      }

      for {set i 1} {$i <=8} {incr i} {
         puts -nonewline $f [binary format i1 -1]     ;#  annotation pointers
      }

      for {set i 1} {$i <=8} {incr i} {
         puts -nonewline $f [binary format i1 -1]     ;#  calibration pointers
      }

      for {set i 1} {$i <=8} {incr i} {
         puts -nonewline $f [binary format i1 -1]     ;#  efficiency pointers
      }

                                                     ;#  data array 1 descriptor 
      puts -nonewline $f [binary format i1 0]        ;#  array layout
      puts -nonewline $f [binary format i1 $type]    ;#  array type  (32 bit unsigned)
      puts -nonewline $f [binary format i1 0]        ;#  reserved 
      puts -nonewline $f [binary format i1 0]        ;#  reserved
      puts -nonewline $f [binary format i1 0]        ;#  spectrum counts offset

                                                     ;#  data array 2 descriptor 
      puts -nonewline $f [binary format i1 0]        ;#  array layout
      puts -nonewline $f [binary format i1 $type]    ;#  array type 
      puts -nonewline $f [binary format i1 0]        ;#  reserved 
      puts -nonewline $f [binary format i1 0]        ;#  reserved
      puts -nonewline $f [binary format i1 -1]       ;#  spectrum counts offset

      puts -nonewline $f [binary format i1 436]      ;#  base of string space
      puts -nonewline $f [binary format i1 20]       ;#  string free space
      puts -nonewline $f [binary format i1 76]       ;#  top of string space

      puts -nonewline $f [binary format i1 512]      ;#  base of counts space
      puts -nonewline $f [binary format i1 $range]   ;#  string free space
      puts -nonewline $f [binary format i1 $range]   ;#  top of string space
 
      puts -nonewline $f [binary format a20 $title]  ;# string space - title

      for {set i 456} {$i < 512} {incr i} {
         puts -nonewline $f [binary format c1 0]     ;#  unused
      }

      puts -nonewline $f [binary format i* $counts]  ;# counts

      close $f
}

    proc HistogramSave {args} {
       global env
       global tcl_platform
       variable SpecBase
       variable SpecPath
       global TempDir

       set fname [file join $TempDir HistogramSave.log]
       set z [catch {open $fname w} fid]
       if {$z != 0} {puts "Histogram Save logfile not created\n$fid\n"}

        puts "Saving  Histograms"
        puts $fid "Saving  Histograms at [clock format [clock seconds] -format "%B %d, %Y %H:%M:%S"]"

      set f1 $SpecBase
      set z1 [catch {file mkdir $f1} m1]
      if {$z1 != 0} then {puts $fid "Cannot create $f1 - $m1"}

      if {$tcl_platform(platform) != "windows"} {
           set z2 [catch {file attributes $f1 -permissions rwxrwxrwx} m2]
           if {$z2 != 0} then {puts "error accessing $f1 - $m2"}
      }

      set f1 [file join $f1 $env(ComputerName)]
      set z1 [catch {file mkdir $f1} m1]
      if {$z1 != 0} then {puts $fid "Cannot create $f1 - $m1"}

      if {$tcl_platform(platform) != "windows"} {
           set z2 [catch {file attributes $f1 -permissions rwxrwxrwx} m2]
           if {$z2 != 0} then {puts "error accessing $f1 - $m2"}
      }

      set f2 [file join $f1 [date +%Y%h%d-%H.%M.%S]]
      set z1 [catch {file mkdir $f2} m1]
      if {$z1 != 0} then {puts $fid "Cannot create $f2 - $m1"}

      if {$tcl_platform(platform) != "windows"} {
           set z2 [catch {file attributes $f2 -permissions rwxrwxrwx} m2]
           if {$z2 != 0} then {puts "error accessing $f2 - $m2"}
      }

       set SpecPath $f2

       set SpecNames [SpectrumNames]
       set NumSpec 0

      foreach item $SpecNames {
         if {[first $item] != 1} {continue}   ;#   not a 1D histogram !!

         set SpecName [second $item]
         set spec [last [split $SpecName .]]

         set info [SpectrumDetails $SpecName]
         set channels [lindex $info 2]

         set title [SpectrumGetTitle $SpecName]
         set counts [SpectrumRead1D $SpecName 0 $channels]

         set z [catch {SpecSave1D $SpecName 0 $channels 5 $title $counts} m]
         if {$z!=0} then {
             puts $fid "Error attempting to save spectrum $SpecName: $m"
         } else {
             incr NumSpec
         }
      }

        puts "$NumSpec Histograms saved"
        puts $fid "$NumSpec Histograms saved at [clock format [clock seconds] -format "%B %d, %Y %H:%M:%S"]"

        close $fid

    }

    proc HistogramZero {} {

       set SpecNames [SpectrumNames]
       set NumSpec 0

       ZeroSpectrum Stat
       incr NumSpec

      foreach item $SpecNames {
         if {[first $item] != 1} {continue}   ;#   not a 1D histogram !!

         set SpecName [second $item]

         ZeroSpectrum $SpecName
         incr NumSpec
      }

       puts "$NumSpec spectra zeroed"
    }


    proc TerminateExec {} {

        unmap_spectra
        DataAcqCommand 22 "Terminating DataAcq program" 0 2

    }

    proc LoadExec {} {
       global Options
       global env

    }

    make-UIregisters

    Set_Options



