# Note:  This is loaded in the namespace
# variable started 0

if {![info exists started]} {variable started 0 }

if {$started != 0} {
    variable JS ""
    do_click
}

if {$started == 0} {           ;#  first time load code and define local variables
    set started 1
    
    variable JS ""
    
    proc do_redisplay {} {;}
    
    proc do_measureEth { params } {
	
	# Set shaping time to max.
	# Set Eth to target E-th value.
	# Loop over pulse width. 
	
	# set name [lindex [lindex $params 1] 0]
	set sideUnderTest [lindex [lindex $params 1] 1] ;# [a.u.]
	set asicUnderTest [lindex [lindex $params 1] 2] ;# [a.u.]
	
	do_getState 
	# Start DAQ.
	after 500 set timeout 1
	do_go
	vwait timeout
	# Mandatory to enquire state
	if { [do_getState] != 1 } {return "Daq not started properly.";}
	
	# Loop over individual side.
	foreach itrSide [set ::R3B::[set ::R3B::R3B4K].SIDEList]  {
	    
	    if {$itrSide != $sideUnderTest} {continue;}
	    
	    # Loop over individual asic.
	    foreach itrAsic [set ::R3B::[set ::R3B::R3B4K].ASICList$itrSide]  {
		
		if {$asicUnderTest != "all" && $asicUnderTest != $itrAsic } {continue;}
		
		# Set file name
		set fileName  /MIDAS/SpecData/R3B/TestData/
		append fileName [clock format [clock seconds] -format "%Y-%m-%d-"]
		append fileName "MeasureEth-"
		append fileName "Side$itrSide-Asic$itrAsic.txt"
		
		# Create file and set permissions.
		set flagOpen [catch {open $fileName w} fileInput]
		set flagPerm [catch {file attributes $fileName -permissions rwxrwxrwx} ]
		if { $flagOpen != 0 } { InsertLog "Cannot open $fileName." }
		if { $flagPerm != 0 } { InsertLog "Error setting permission of $fileName." }
		
		# Enable test pulse injection
		set_asicValidate $itrSide $itrAsic 1
		
		# Set Eth.
		set registerEth 15
		WriteR3BASICRegister [expr ($itrSide << 18) + ($itrAsic << 10) + $registerEth] 133
		
		# Set shaping time.
		
		# Set pulse.
		set pulseWidth 10 
		set pulseDelay 10000 
		set pulseNumber 512
		
		do_loadPulseParams  $pulseWidth $pulseDelay $pulseNumber 
		
		# Loop over individual channels.
		set channelNumber 128
		set channelGroup 8
		for {set itrGroup 0} {$itrGroup < $channelNumber/$channelGroup} {incr itrGroup} {
		    
		    if {$itrGroup !=0 } { continue;}
		    
		    # Power ON channels.
		    for {set itrChannel 0} {$itrChannel < $channelGroup} {incr itrChannel} {
			set dummyChannel [expr $itrGroup + ($itrChannel* ($channelNumber/$channelGroup) ) ]
			set valuePwr [expr 255 - int( pow (2,  $dummyChannel - ( $dummyChannel/8)*8 ) ) ]
			set registerPwr [expr int ( 80 + ($dummyChannel/8) )]
			WriteR3BASICRegister [expr ($itrSide << 18) + ($itrAsic << 10) + $registerPwr] $valuePwr
		    } 
		    
		    set vcall 64 ;# [DN]
		    set registerVcall 17
		    WriteR3BASICRegister [expr ($itrSide << 18) + ($itrAsic << 10) + $registerVcall] $vcall
		    
		    set vcalhStep 10;# [DN]
		    set vcalhMin [expr $vcall + $vcalhStep];# [DN]
		    set vcalhMax 160; #[DN]
		    # Loop over pulse width.
		    for {set itrVcalh $vcalhMin} {$itrVcalh <= $vcalhMax} {set itrVcalh [expr $itrVcalh+$vcalhStep]} {
			
			set registerVcalh 16
			WriteR3BASICRegister [expr ($itrSide << 18) + ($itrAsic << 10) + $registerVcalh] $itrVcalh
			
			# Inject pulses.
			do_testPulse $pulseWidth $pulseDelay $pulseNumber
			
			# Zero histograms.
			for {set itrChannel 0} {$itrChannel < $channelGroup} {incr itrChannel} {
			    set dummyChannel [expr $itrGroup + ($itrChannel* ($channelNumber/$channelGroup) ) ]
			    set fileName "E.$itrSide.$itrAsic.$dummyChannel"
			    # Empty the content of the histogram 
			    # Check also that the histogram has been created.
			    if { [ZeroSpectrum $fileName] != 0 } { 
				return "Histograms E.$itrSide.$itrAsic.* were not created";}
			}
			
			# Inject pulses.
			do_testPulse $pulseWidth $pulseDelay $pulseNumber
			
			# Dump data.
			for {set itrChannel 0} {$itrChannel < $channelGroup} {incr itrChannel} {
			    set dummyChannel [expr $itrGroup + ($itrChannel* ($channelNumber/$channelGroup) ) ]
			    set plot  [do_analyseSpectra $itrSide $itrAsic $dummyChannel]
			    set range [lindex $plot 0]
			    set data  [lindex $plot 1]
			    puts $fileInput "$itrSide $itrAsic $dummyChannel $itrVcalh $pulseNumber $range"
			    puts $fileInput $data
			    flush $fileInput
			}
			
		    } ;# end loop over vcalh. 
		    
		    # Power OFF channel.
		    for {set itrChannel 0} {$itrChannel < $channelGroup} {incr itrChannel} {
			set dummyChannel [expr $itrGroup + ($itrChannel* ($channelNumber/$channelGroup) ) ]
			set registerPwr [expr int ( 80 + ($dummyChannel/8) )]
			WriteR3BASICRegister [expr ($itrSide << 18) + ($itrAsic << 10) + $registerPwr] 255
		    }
		    
		} ;# end of loop over group.
		
		close $fileInput
		# Disable test pulse injection
		set_asicValidate $itrSide $itrAsic 0 
		
	    } ;# end of loop over asic.
	} ;# end of loop over side.
	
	# Stop DAQ
	do_getState
	after 500 set timeout 1
	do_stop
	vwait timeout
	# Mandatory to enquire state
	if { [do_getState] != 0 } {return "Daq did not stop correctly";}
	
	InsertLog "do_MeasureEth"
	
    }
    
    proc do_tuneEthCoarse { params } { 
	
	variable msgIden
	
	# set name [lindex [lindex $params 1] 0]
	set sideUnderTest [lindex [lindex $params 1] 1] ;# [a.u.]
	set asicUnderTest [lindex [lindex $params 1] 2] ;# [a.u.]
	set offsetMin -64 ;# [DAC]
	set offsetMax  64 ;# [DAC]
	set offsetStep 2  ;# [DAC]
	set deltaT 1000; # [ms] 
	
	do_getState
	# Start DAQ.
	after 5000 set timeout 1
	do_go
	vwait timeout
	# Mandatory to enquire state.
	if { [do_getState] != 1 } {return "Daq not started properly.";}
	
	# Loop over individual side.
	foreach itrSide [set ::R3B::[set ::R3B::R3B4K].SIDEList]  {
	    
	    if {$itrSide != $sideUnderTest} {continue;}
	    
	    # Loop over individual asic.
	    foreach itrAsic [set ::R3B::[set ::R3B::R3B4K].ASICList$itrSide]  {
		
		if {$asicUnderTest != "all" && $asicUnderTest != $itrAsic } {continue;}
		
		# --
		# TO BE REMOVED.
		# Loop over PkTm and load offsets.
		foreach itrPkTm {14 126 254} {
		    # --
		    
		    # # -- 
		    # # TO BE RESTORED
		    # # Set file name
		    # set fileName  /MIDAS/SpecData/R3B/TestData/
		    # append fileName [clock format [clock seconds] -format "%Y-%m-%d-"]
		    # append fileName "TuneEthCoarse-"
		    # append fileName "Side$itrSide-Asic$itrAsic.txt" 
		    # # --
		    
		    # --
		    # TO BE REMOVED.
		    # Set PkTm.
		    set registerPkTm 4
		    WriteR3BASICRegister [expr ($itrSide << 18) + ($itrAsic << 10) + $registerPkTm] $itrPkTm 
		    # Set file name
		    set fileName  /MIDAS/SpecData/R3B/TestData/
		    append fileName [clock format [clock seconds] -format "%Y-%m-%d-"]
		    append fileName "TuneEthCoarse-"
		    append fileName "Side$itrSide-Asic$itrAsic-Reg4-$itrPkTm.txt"
		    # --
		    
		    # Create file and set permissions.
		    set flagOpen [catch {open $fileName w} fileInput]
		    set flagPerm [catch {file attributes $fileName -permissions rwxrwxrwx} ]
		    if { $flagOpen != 0 } { InsertLog "Cannot open $fileName." }
		    if { $flagPerm != 0 } { InsertLog "Error setting permission of $fileName." }
		    
		    # Flush file header.
		    puts $fileInput "$itrSide $itrAsic $offsetMin $offsetMax $offsetStep"
		    flush $fileInput
		    
		    # Set Eth to minimum (0x80)
		    set registerEth 15
		    WriteR3BASICRegister [expr ($itrSide << 18) + ($itrAsic << 10) + $registerEth] 0x80
		    
		    # Loop over individual channels.
		    set channelNumber 128
		    set channelIncrease 1
		    for {set itrChannel 0} {$itrChannel < $channelNumber} {set itrChannel [expr $itrChannel+$channelIncrease]} {
			
			# if { $itrChannel != 0 } { continue;}
			
			set valuePwr [expr 255 - int( pow (2,  $itrChannel - ( $itrChannel/8)*8 ) ) ]
			set registerPwr [expr 80 + ($itrChannel/8)]
			WriteR3BASICRegister [expr ($itrSide << 18) + ($itrAsic << 10) + $registerPwr] $valuePwr
			
			for {set itrOffset $offsetMin} {$itrOffset < 0} {set itrOffset [expr $itrOffset+$offsetStep]} {
			    set offsetValue [expr 127 + abs($itrOffset)]
			    set page 3
			    WriteR3BASICRegister [expr ($itrSide << 18) + ($itrAsic << 10) + ($page << 8) + (128+$itrChannel) ] $offsetValue  
			    set valuePwr [expr 255 - int( pow (2,  $itrChannel - ( $itrChannel/8)*8 ) ) ]
			    set registerPwr [expr 80 + ($itrChannel/8)]
			    WriteR3BASICRegister [expr ($itrSide << 18) + ($itrAsic << 10) + $registerPwr] $valuePwr
			    
			    # Wait for power up.
			    if {$itrOffset == $offsetMin} {after $deltaT}
			    
			    # Zero histo.
			    set fileName "E.$itrSide.$itrAsic.$itrChannel"
			    # This zeros the histogram.
			    if { [ZeroSpectrum $fileName] != 0 } { 
			    	return "Histograms E.$itrSide.$itrAsic.* were not created";}
			    
			    # Wait.
			    after $deltaT
			    
			    set plot  [do_analyseSpectra $itrSide $itrAsic $itrChannel]
			    set range [lindex $plot 0]
			    set data  [lindex $plot 1]
			    
			    puts $fileInput "$itrChannel $itrOffset $range"
			    puts $fileInput $data
			    flush $fileInput
			    
			    # Power off channel.
			    WriteR3BASICRegister [expr ($itrSide << 18) + ($itrAsic << 10) + $registerPwr] 255
			    
			} ;# end of loop over offset.
			
			for {set itrOffset 0} {$itrOffset < [expr $offsetMax +$offsetStep]} {set itrOffset [expr $itrOffset+$offsetStep]} {
			    
			    set offsetValue $itrOffset
			    set page 3
			    WriteR3BASICRegister [expr ($itrSide << 18) + ($itrAsic << 10) + ($page << 8) + (128+$itrChannel) ] $offsetValue 
			    set valuePwr [expr 255 - pow (2, $itrChannel - ($itrChannel/8)*8 ) ]
			    set registerPwr [expr 80 + ($itrChannel/8)]
			    WriteR3BASICRegister [expr ($itrSide << 18) + ($itrAsic << 10) + $registerPwr] $valuePwr
			    # Zero histo.
			    set fileName "E.$itrSide.$itrAsic.$itrChannel"
			    # This zeros the histogram.
			    if { [ZeroSpectrum $fileName] != 0 } { 
			    	return "Histograms E.$itrSide.$itrAsic.* were not created";}
			    
			    # Wait.
			    after $deltaT
			    
			    set plot  [do_analyseSpectra $itrSide $itrAsic $itrChannel]
			    set range [lindex $plot 0]
			    set data  [lindex $plot 1]
			    
			    puts $fileInput "$itrChannel $itrOffset $range"
			    puts $fileInput $data
			    flush $fileInput
			    
			    # Power off channel.
			    WriteR3BASICRegister [expr ($itrSide << 18) + ($itrAsic << 10) + $registerPwr] 255
			} ;# end loop over offset
			
		    }  ;# end of loop over channel.
		    
		    # --
		    # TO BE REMOVED.
		} ;# end of foreach
		#--
		
		close $fileInput
		
	    } ;# end of loop over asic.
	}  ;#end of loop over side.
	
	do_getState
	# Stop DAQ
	after 5000 set timeout 1
	do_stop
	vwait timeout
	# Mandatory to enquire state
	if { [do_getState] != 0 } { 
	    #puts "$msgIden Daq did not stop correctly going"; 
	    return "Daq did not stop correctly"
	}
	
	return "Measurement completed" 
	
    }
    
    proc do_tuneEthCoarseBis { params } {
	
	variable msgIden
	
	# set name [lindex [lindex $params 1] 0]
	set sideUnderTest [lindex [lindex $params 1] 1] ;# [a.u.]
	set asicUnderTest [lindex [lindex $params 1] 2] ;# [a.u.]
	set pulseNumber   2048;# [lindex [lindex $params 1] 3] ;# [a.u.]
	set pulseWidth    1000;# [lindex [lindex $params 1] 4] ;# [10ns]
	set pulseDelay    50000;# [lindex [lindex $params 1] 5] ;# [10ns]
	
	do_getState
	# Start DAQ.
	after 5000 set timeout 1
	do_go
	vwait timeout
	# Mandatory to enquire state
	if { [do_getState] != 1 } {return "Daq not started properly.";}
	
	# Loop over individual side.
	foreach itrSide [set ::R3B::[set ::R3B::R3B4K].SIDEList]  {
	    
	    if {$itrSide != $sideUnderTest} {continue;}
	    
	    # Loop over individual asic.
	    foreach itrAsic [set ::R3B::[set ::R3B::R3B4K].ASICList$itrSide]  {
		
	  	if {$asicUnderTest != "all" && $asicUnderTest != $itrAsic } {continue;}
		
	  	# Set file name
	  	set fileName  /MIDAS/SpecData/R3B/TestData/
	  	append fileName [clock format [clock seconds] -format "%Y-%m-%d-"]
	  	append fileName "TuneEthCoarseBis-"
	  	append fileName "Side$itrSide-Asic$itrAsic.txt"
		
	 	# Create file and set permissions.
	  	set flagOpen [catch {open $fileName w} fileInput]
	  	set flagPerm [catch {file attributes $fileName -permissions rwxrwxrwx} ]
	  	if { $flagOpen != 0 } { InsertLog "Cannot open $fileName." }
	  	if { $flagPerm != 0 } { InsertLog "Error setting permission of $fileName." }
		
	 	# Enable test pulse injection
	  	set_asicValidate $itrSide $itrAsic 1
		# Set pulse params
		do_loadPulseParams $pulseWidth $pulseDelay $pulseNumber
		
	 	# Set pulse amplitude.
	 	set voltStep [expr 2.0/256] ;# [V/DN] 
		
	 	set vcall 0.5 ;# [V]
	 	set valueVcall [expr round( ($vcall)/$voltStep ) ] ;# [DN]
		set registerVcall 17
	 	WriteR3BASICRegister [expr ($itrSide << 18) + ($itrAsic << 10) + $registerVcall] $valueVcall
		
	 	set valueVcalh [expr 3 + $valueVcall] ;# [DN]
	 	set registerVcalh 16
	 	WriteR3BASICRegister [expr ($itrSide << 18) + ($itrAsic << 10) + $registerVcalh] $valueVcalh
		
		# Set Eth.
		set registerEth 15
		WriteR3BASICRegister [expr ($itrSide << 18) + ($itrAsic << 10) + $registerEth] 170
		
		# Loop over offset.
		set skipOffset 0
		for {set itrOffset 128} {$itrOffset < 255+1} {set itrOffset [expr $itrOffset+2] } {
		    
		    # Set offset.
		    set page 3
		    for {set itrChannel 0} {$itrChannel < 128} {incr itrChannel} {
			WriteR3BASICRegister [expr ($itrSide << 18) + ($itrAsic << 10) + ($page << 8) + (128+$itrChannel) ] $itrOffset 
		    }
		    
		    # Wait to switch ON.
		    after 500 ;# [ms]
		    
		    # Loop over individual channels.
		    set channelNumber 128
		    set channelGroup 16
		    for {set itrGroup 0} {$itrGroup < $channelNumber/$channelGroup} {incr itrGroup} {
			
			if {$itrGroup !=0 } { continue;}
			
			# Power ON channels.
			for {set itrChannel 0} {$itrChannel < $channelGroup} {incr itrChannel} {
			    set dummyChannel [expr $itrGroup + ($itrChannel* ($channelNumber/$channelGroup) ) ]
			    set valuePwr [expr 255 - int( pow (2,  $dummyChannel - ( $dummyChannel/8)*8 ) ) ]
			    set registerPwr [expr int ( 80 + ($dummyChannel/8) )]
			    WriteR3BASICRegister [expr ($itrSide << 18) + ($itrAsic << 10) + $registerPwr] $valuePwr
			} 
			
			# Inject pulses.
			do_testPulse $pulseWidth $pulseDelay $pulseNumber
			
			# Zero histograms.
			for {set itrChannel 0} {$itrChannel < $channelGroup} {incr itrChannel} {
			    set dummyChannel [expr $itrGroup + ($itrChannel* ($channelNumber/$channelGroup) ) ]
			    set fileName "E.$itrSide.$itrAsic.$dummyChannel"
			    # Empty the content of the histogram 
			    # Check also that the histogram has been created.
			    if { [ZeroSpectrum $fileName] != 0 } { 
				return "Histograms E.$itrSide.$itrAsic.* were not created";}
			}
			
			# Inject pulses.
			do_testPulse $pulseWidth $pulseDelay $pulseNumber
			
			# Dump data.
			for {set itrChannel 0} {$itrChannel < $channelGroup} {incr itrChannel} {
			    set dummyChannel [expr $itrGroup + ($itrChannel* ($channelNumber/$channelGroup) ) ]
			    set plot  [do_analyseSpectra $itrSide $itrAsic $dummyChannel]
			    set range [lindex $plot 0]
			    set data  [lindex $plot 1]
			    set offsetLabel $itrOffset ;#[expr (127-$itrOffset)]
			    puts $fileInput "$itrSide $itrAsic $dummyChannel $offsetLabel $pulseNumber $range"
			    puts $fileInput $data
			    flush $fileInput
			}
			
			# Power OFF channel.
			for {set itrChannel 0} {$itrChannel < $channelGroup} {incr itrChannel} {
			    set dummyChannel [expr $itrGroup + ($itrChannel* ($channelNumber/$channelGroup) ) ]
			    set registerPwr [expr int ( 80 + ($dummyChannel/8) )]
			    WriteR3BASICRegister [expr ($itrSide << 18) + ($itrAsic << 10) + $registerPwr] 255
			}
			
		    } ;# end loop over group.
		    
		} ;# end of loop over offset.
		
		close $fileInput
	  	# Disable test pulse injection
	  	set_asicValidate $itrSide $itrAsic 0 
		
	    } ;# end of loop over asic.
	} ;# end of loop over side.
	
	# Stop DAQ
	after 5000 set timeout 1
	do_stop
	vwait timeout
	# Mandatory to enquire state
	if { [do_getState] != 0 } { puts "$msgIden Daq did not stop correctly going"; 
	    return "Daq did not stop correctly";}
	
	return "Test complete"
    }
    
    proc do_tuneEthCoarseTer { params } {
	
	# Set shaping time to max.
	# Set Eth to target E-th value.
	# Loop over pulse width.
	# Change offset. 
	# Repeat loop over pulse width.
	
	# set name [lindex [lindex $params 1] 0]
	set sideUnderTest [lindex [lindex $params 1] 1] ;# [a.u.]
	set asicUnderTest [lindex [lindex $params 1] 2] ;# [a.u.]
	
	# Loop over individual side.
	foreach itrSide [set ::R3B::[set ::R3B::R3B4K].SIDEList]  {
	    
	    if {$itrSide != $sideUnderTest} {continue;}
	    
	    # Loop over individual asic.
	    foreach itrAsic [set ::R3B::[set ::R3B::R3B4K].ASICList$itrSide]  {
		
	  	if {$asicUnderTest != "all" && $asicUnderTest != $itrAsic } {continue;}
		
		# Set file name
	  	set fileName  /MIDAS/SpecData/R3B/TestData/
	  	append fileName [clock format [clock seconds] -format "%Y-%m-%d-"]
	  	append fileName "TuneEthCoarseTer-"
	  	append fileName "Side$itrSide-Asic$itrAsic.txt"
		
	 	# Create file and set permissions.
	  	set flagOpen [catch {open $fileName w} fileInput]
	  	set flagPerm [catch {file attributes $fileName -permissions rwxrwxrwx} ]
	  	if { $flagOpen != 0 } { InsertLog "Cannot open $fileName." }
	  	if { $flagPerm != 0 } { InsertLog "Error setting permission of $fileName." }
		
	 	# Enable test pulse injection
	  	set_asicValidate $itrSide $itrAsic 1
		# Set pulse params
		do_loadPulseParams $pulseWidth $pulseDelay $pulseNumber
		
		set valueVcall 64 ;# [DN]
		set registerVcall 17
	 	WriteR3BASICRegister [expr ($itrSide << 18) + ($itrAsic << 10) + $registerVcall] $valueVcall
		
	 	set valueVcalh [expr 2 + $valueVcall] ;# [DN]
	 	set registerVcalh 16
	 	WriteR3BASICRegister [expr ($itrSide << 18) + ($itrAsic << 10) + $registerVcalh] $valueVcalh
		
		# Set Eth.
		set registerEth 15
		WriteR3BASICRegister [expr ($itrSide << 18) + ($itrAsic << 10) + $registerEth] 133
		
		set pulseDelay 10000 
		set pulseNumber 512
		
		# Loop over pulse width.
		for {set itrPulseWidth 0} {$itrPulseWidth < 500} {set itrPulseWidth [expr $itrPulseWidth+5]} {
		    
		    # Set pulse params
		    # do_loadPulseParams $itrPulseWidth $pulseDelay $pulseNumber 
		    do_loadPulseParams  $itrPulseWidth $pulseDelay $pulseNumber 
		    
		    # Loop over individual channels.
		    set channelNumber 128
		    set channelGroup 16
		    for {set itrGroup 0} {$itrGroup < $channelNumber/$channelGroup} {incr itrGroup} {
			
			# if {$itrGroup !=0 } { continue;}
			
			# Power ON channels.
			for {set itrChannel 0} {$itrChannel < $channelGroup} {incr itrChannel} {
			    set dummyChannel [expr $itrGroup + ($itrChannel* ($channelNumber/$channelGroup) ) ]
			    set valuePwr [expr 255 - int( pow (2,  $dummyChannel - ( $dummyChannel/8)*8 ) ) ]
			    set registerPwr [expr int ( 80 + ($dummyChannel/8) )]
			    WriteR3BASICRegister [expr ($itrSide << 18) + ($itrAsic << 10) + $registerPwr] $valuePwr
			} 
			
			# Inject pulses.
			do_testPulse $itrPulseWidth $pulseDelay $pulseNumber
			
			# Zero histograms.
			for {set itrChannel 0} {$itrChannel < $channelGroup} {incr itrChannel} {
			    set dummyChannel [expr $itrGroup + ($itrChannel* ($channelNumber/$channelGroup) ) ]
			    set fileName "E.$itrSide.$itrAsic.$dummyChannel"
			    # Empty the content of the histogram 
			    # Check also that the histogram has been created.
			    if { [ZeroSpectrum $fileName] != 0 } { 
				return "Histograms E.$itrSide.$itrAsic.* were not created";}
			}
			
			# Inject pulses.
			do_testPulse $itrPulseWidth $pulseDelay $pulseNumber
			
			# Dump data.
			for {set itrChannel 0} {$itrChannel < $channelGroup} {incr itrChannel} {
			    set dummyChannel [expr $itrGroup + ($itrChannel* ($channelNumber/$channelGroup) ) ]
			    set plot  [do_analyseSpectra $itrSide $itrAsic $dummyChannel]
			    set range [lindex $plot 0]
			    set data  [lindex $plot 1]
			    puts $fileInput "$itrSide $itrAsic $dummyChannel $itrPulseWidth $pulseNumber $range"
			    puts $fileInput $data
			    flush $fileInput
			}
			
			# Power OFF channel.
			for {set itrChannel 0} {$itrChannel < $channelGroup} {incr itrChannel} {
			    set dummyChannel [expr $itrGroup + ($itrChannel* ($channelNumber/$channelGroup) ) ]
			    set registerPwr [expr int ( 80 + ($dummyChannel/8) )]
			    WriteR3BASICRegister [expr ($itrSide << 18) + ($itrAsic << 10) + $registerPwr] 255
			}
			
		    } ;# end of loop over group.
		    
		} ;# end loop over pulse width. 
		
		close $fileInput
	  	# Disable test pulse injection
	  	set_asicValidate $itrSide $itrAsic 0 
		
	    } ;# end of loop over asic.
	} ;# end of loop over side.
	
	InsertLog "do_tuneEthCoarseTer";
	
    }
    
    proc do_tuneEthFine { params } {
	
	variable msgIden
	
	# set name [lindex [lindex $params 1] 0]
	set sideUnderTest [lindex [lindex $params 1] 1]
	set asicUnderTest [lindex [lindex $params 1] 2]
	
	do_getState
	# Start DAQ.
	after 5000 set timeout 1
	do_go
	vwait timeout
	# Mandatory to enquire state.
	if { [do_getState] != 1 } {return "Daq not started properly.";}
	
	# Loop over individual side.
	foreach itrSide [set ::R3B::[set ::R3B::R3B4K].SIDEList]  {
	    
	    if {$itrSide != $sideUnderTest} {continue;}
	    
	    # Loop over individual asic.
	    foreach itrAsic [set ::R3B::[set ::R3B::R3B4K].ASICList$itrSide]  {
		
		if {$asicUnderTest != "all" && $asicUnderTest != $itrAsic } {continue;}
		
	 	# Set file name
	 	set fileName  /MIDAS/SpecData/R3B/TestData/
	 	append fileName [clock format [clock seconds] -format "%Y-%m-%d-"]
	 	append fileName "TuneEthFine-"
	 	append fileName "Side$itrSide-Asic$itrAsic.txt"
		# Create file and set permissions.
	 	set flagOpen [catch {open $fileName w} fileInput]
	 	set flagPerm [catch {file attributes $fileName -permissions rwxrwxrwx} ]
	 	if { $flagOpen != 0 } { InsertLog "Cannot open $fileName." }
	 	if { $flagPerm != 0 } { InsertLog "Error setting permission of $fileName." }
		
	 	# Set Eth to minimum (0x80)
	 	set registerEth 15
	 	WriteR3BASICRegister [expr ($itrSide << 18) + ($itrAsic << 10) + $registerEth] 0x80
		
		# Loop over individual channels.
	 	set channelNumber 128
	 	set channelIncrease 1
		for {set itrChannel 0} {$itrChannel < $channelNumber} {set itrChannel [expr $itrChannel+$channelIncrease]} {
		    # if { $itrChannel != 63 } { continue;}
		    
		    set valuePwr [expr 255 - int( pow (2,  $itrChannel - ( $itrChannel/8)*8 ) ) ]
		    set registerPwr [expr 80 + ($itrChannel/8)]
		    WriteR3BASICRegister [expr ($itrSide << 18) + ($itrAsic << 10) + $registerPwr] $valuePwr
		    
		    set page 3
		    set coarseOffset [expr [ReadR3BASICRegister [expr ($itrSide << 18) + ($itrAsic << 10) + ($page << 8) + (128+$itrChannel) ] ] ]
		    
		    variable coarseOffsetValue 
		    if { $coarseOffset>=0   && $coarseOffset<=127} { set coarseOffsetValue $coarseOffset }
		    if { $coarseOffset>=128 && $coarseOffset<=255} { set coarseOffsetValue [expr 127-$coarseOffset]}
		    
		    set flagFirstIteration 1
		    set rangeOffset 5
		    for {set itrOffset -$rangeOffset} {$itrOffset <= +$rangeOffset} {incr itrOffset} {
			
			# Check for out-of-range.
			if { [expr $coarseOffsetValue + $itrOffset]< -128 || 
			     [expr $coarseOffsetValue + $itrOffset]> 127 } {
			    InsertLog "Offset=$itrOffset is out of range in Side:$itrSide Asic:$itrAsic Channel:$itrChannel";
			    continue;
			}
			
			# Set appropriate value.
			variable offsetValue
			if { [expr $coarseOffsetValue + $itrOffset] >= 0} { 
			    set offsetValue [expr $coarseOffsetValue + $itrOffset]
			} elseif {[expr $coarseOffsetValue + $itrOffset] < 0} { 
			    set offsetValue [expr 127 + abs([expr $coarseOffsetValue + $itrOffset])]
			}
			
			set page 3
			WriteR3BASICRegister [expr ($itrSide << 18) + ($itrAsic << 10) + ($page << 8) + (128+$itrChannel) ] $offsetValue 
			
			set valuePwr [expr 255 - pow (2, $itrChannel - ($itrChannel/8)*8 ) ]
			set registerPwr [expr 80 + ($itrChannel/8)]
			WriteR3BASICRegister [expr ($itrSide << 18) + ($itrAsic << 10) + $registerPwr] $valuePwr
			
			# Wait for power up.
			if {$flagFirstIteration==1} {
			    after 500; 
			    set flagFirstIteration 0
			} else {after 100;}
			
			# Zero histo.
			set fileName "E.$itrSide.$itrAsic.$itrChannel"
			# This zeros the histogram.
			if { [ZeroSpectrum $fileName] != 0 } { 
		     	    return "Histograms E.$itrSide.$itrAsic.* were not created";}
			
			# Wait.
			after 100
			
			set plot  [do_analyseSpectra $itrSide $itrAsic $itrChannel]
			set range [lindex $plot 0]
			set data  [lindex $plot 1]
			
			puts $fileInput "$itrSide $itrAsic $itrChannel $offsetValue $itrOffset $range"
			puts $fileInput $data
			flush $fileInput
			
			# Power off channel.
			WriteR3BASICRegister [expr ($itrSide << 18) + ($itrAsic << 10) + $registerPwr] 255
		    }
		    
		    # Set offset to initial value.
		    WriteR3BASICRegister [expr ($itrSide << 18) + ($itrAsic << 10) + ($page << 8) + (128+$itrChannel) ] $coarseOffset
		    
		} ;# end loop over channel.
		
		close $fileInput
	    } ;# end of loop over asic.
	}  ;#end of loop over side.
	
	do_getState
	# Stop DAQ
	after 5000 set timeout 1
	do_stop
	vwait timeout
	
	# Mandatory to enquire state
	if { [do_getState] != 0 } { 
	    #puts "$msgIden Daq did not stop correctly going"; 
	    return "Daq did not stop correctly"
	}
	
	return "Measurement completed"
	
    }
    
    proc do_loadOffset { params } {
	
	variable msgIden
	
	## -- 
	## TO BE RESTORED
	# set name [lindex [lindex $params 1] 0]
	# set fileName [lindex [lindex $params 1] 1]
	## --
	
	# -- 
	# TO BE REMOVED.
	set fileName $params
	# --
		
	# Open file.
	set f [open $fileName]
	
	# Get header.
	variable side
	variable asic
	if {[gets $f  line] >= 0} {
	    set side    [lindex $line 0] ;
	    set asic    [lindex $line 1] ;
	}
	
	# Get body.
	while {[gets $f  line] >= 0} {
	    set channel [lindex $line 0]
	    set offset  [lindex $line 1]
	    
	    # set offset   $max;# [expr round($mean-0*$stdDev)]
	    
	    if {$offset >= 0} {
		set page 3
		WriteR3BASICRegister [expr ($side << 18) + ($asic << 10) + ($page << 8) + (128+$channel) ] $offset  
	    } elseif {$offset < 0} {
		set page 3
	 	WriteR3BASICRegister [expr ($side << 18) + ($asic << 10) + ($page << 8) + (128+$channel) ] [expr abs($offset) + 127];
	    }
	    
	}
	
	return $fileName
    }
    
    ## --
    ## TO BE REMOVED
    ## --
    proc do_loadOffsetPg1 { side asic offset } {
    	
	# set name [lindex [lindex $params 1] 0]
	##set side [lindex [lindex $params 1] 1]	
	##set asic [lindex [lindex $params 1] 2]
	## set offset [lindex [lindex $params 1] 3]
	set page 1
	
	for {set itrChannel 0} {$itrChannel < 128} {incr itrChannel} {
	    
	    if {$offset >= 0} {
		
		WriteR3BASICRegister [expr ($side << 18) + ($asic << 10) + ($page << 8) + (128+$itrChannel) ] $offset  
	    } elseif {$offset < 0} {
		WriteR3BASICRegister [expr ($side << 18) + ($asic << 10) + ($page << 8) + (128+$itrChannel) ] [expr abs($offset) + 127];
	    }
	    
	}
	
	return "Side$side Asic$asic Page$page offsets set to $offset";
    }
    ## --
    
    proc do_noiseFloor {params} {
	
	variable msgIden
	
	# set name [lindex [lindex $params 1] 0]
	set sideUnderTest [lindex [lindex $params 1] 1]
	set asicUnderTest [lindex [lindex $params 1] 2] 
	set ethMin 118 ;# [DAC]
	set ethMax 138 ;# [DAC]
	set ethStep 1  ;# [DAC]
	set deltaT 500 ;# [ms]
	
	# Start DAQ.
	do_getState
	after 500 set timeout 1
	do_go
	vwait timeout
	# Mandatory to enquire state
	if {[do_getState] != 1} {return "DAQ did not start" } 
	
	# Loop over individual side.
	foreach itrSide [set ::R3B::[set ::R3B::R3B4K].SIDEList]  {
	    
	    if {$itrSide != $sideUnderTest} {continue;}
	    
	    # Loop over individual asic.
	    foreach itrAsic [set ::R3B::[set ::R3B::R3B4K].ASICList$itrSide]  {
		
		if {$asicUnderTest != "all" && $asicUnderTest != $itrAsic } {continue;}
		
		# --
		# TO BE REMOVED.
		# Loop over PkTm and load offsets.
		foreach itrPkTm {14 126 254} {
		    foreach itrPg1Offset {-50 0 +50} {
			# --   
			
			# # --
			# # TO BE RESTORED.
			# # Set file name
			# set fileName  /MIDAS/SpecData/R3B/TestData/
			# append fileName [clock format [clock seconds] -format "%Y-%m-%d-"]
			# append fileName "NoiseFloor-"
			# append fileName "Side$itrSide-Asic$itrAsic.txt"
			# # -- 
			
			# --
			# TO BE REMOVED.
			# Set PkTm.
			set registerPkTm 4
			WriteR3BASICRegister [expr ($itrSide << 18) + ($itrAsic << 10) + $registerPkTm] $itrPkTm 
			# Set file name
			set fileName  /MIDAS/SpecData/R3B/TestData/
			append fileName [clock format [clock seconds] -format "%Y-%m-%d-"]
			append fileName "NoiseFloor-"
			append fileName "Side$itrSide-Asic$itrAsic-Reg4-$itrPkTm-Pg1-$itrPg1Offset.txt"
			
			# Load tuning.
			set path /MIDAS/SpecData/R3B/TestData/calibrationFiles/asicV3/B30/
			if {$itrAsic==1 && $itrPkTm==14} {
			    append path 2015-06-29-TuneEthCoarse-Side1-Asic1-Reg4-14-offset.txt
			} elseif {$itrAsic==1 && $itrPkTm==126} {
			    append path 2015-06-29-TuneEthCoarse-Side1-Asic1-Reg4-126-offset.txt
			} elseif {$itrAsic==1 && $itrPkTm==254} {
			    append path 2015-06-29-TuneEthCoarse-Side1-Asic1-Reg4-254-offset.txt
			} elseif {$itrAsic==5 && $itrPkTm==14} {
			    append path 2015-06-29-TuneEthCoarse-Side1-Asic5-Reg4-14-offset.txt
			} elseif {$itrAsic==5 && $itrPkTm==126} {
			    append path 2015-06-29-TuneEthCoarse-Side1-Asic5-Reg4-126-offset.txt
			} elseif {$itrAsic==5 && $itrPkTm==254} {
			    append path 2015-06-29-TuneEthCoarse-Side1-Asic5-Reg4-254-offset.txt
			} elseif {$itrAsic==9 && $itrPkTm==14} {
			    append path 2015-06-30-TuneEthCoarse-Side1-Asic9-Reg4-14-offset.txt
			} elseif {$itrAsic==9 && $itrPkTm==126} {
			    append path 2015-06-30-TuneEthCoarse-Side1-Asic9-Reg4-126-offset.txt
			} elseif {$itrAsic==9 && $itrPkTm==254} {
			    append path 2015-06-30-TuneEthCoarse-Side1-Asic9-Reg4-254-offset.txt
			} elseif {$itrAsic==13 && $itrPkTm==14} {
			    append path 2015-06-30-TuneEthCoarse-Side1-Asic13-Reg4-14-offset.txt
			} elseif {$itrAsic==13 && $itrPkTm==126} {
			    append path 2015-06-30-TuneEthCoarse-Side1-Asic13-Reg4-126-offset.txt
			} elseif {$itrAsic==13 && $itrPkTm==254} {
			    append path 2015-06-30-TuneEthCoarse-Side1-Asic13-Reg4-254-offset.txt
			}
			InsertLog "[do_loadOffset $path]"
			InsertLog "[do_loadOffsetPg1 $itrSide $itrAsic $itrPg1Offset]"
			# --
			
			# Create file and set permissions.
			set flagOpen [catch {open $fileName w} fileInput]
			set flagPerm [catch {file attributes $fileName -permissions rwxrwxrwx} ]
			if { $flagOpen != 0 } { InsertLog "Cannot open $fileName." }
			if { $flagPerm != 0 } { InsertLog "Error setting permission of $fileName." }
			
			# Flush file header.
			puts $fileInput "$itrSide $itrAsic $ethMin $ethMax $ethStep"
			flush $fileInput
						
			# Zero histograms.
			set channelNumber 128
			for {set itrChannel 0} {$itrChannel < $channelNumber} {set itrChannel [expr $itrChannel+8]} {
			    set fileName "E.$itrSide.$itrAsic.$itrChannel"
			    # This should zero the histogram.
			    ZeroSpectrum $fileName
			}
			
			# Loop over Eth from higher to lower value.
			for {set itrEth $ethMax} {$itrEth > $ethMin} {set itrEth [expr $itrEth-$ethStep]} { 
			    
			    # Set Eth.
			    set registerEth 15
			    WriteR3BASICRegister [expr ($itrSide << 18) + ($itrAsic << 10) + $registerEth] $itrEth
			    
			    # Loop over individual channels.
			    set channelNumber 128
			    set channelGroup 8
			    for {set itrGroup 0} {$itrGroup < $channelNumber/$channelGroup} {incr itrGroup} {
				
				# if {$itrGroup >= 1 } { continue;}
				
				# Power ON channels.
				for {set itrChannel 0} {$itrChannel < $channelGroup} {incr itrChannel} {
				    set dummyChannel [expr $itrGroup + ($itrChannel* ($channelNumber/$channelGroup) ) ]
				    set valuePwr [expr 255 - int( pow (2,  $dummyChannel - ( $dummyChannel/8)*8 ) ) ]
				    set registerPwr [expr int ( 80 + ($dummyChannel/8) )]
				    WriteR3BASICRegister [expr ($itrSide << 18) + ($itrAsic << 10) + $registerPwr] $valuePwr
				}
				
				# Wait to switch ON.
				after $deltaT ;# [ms]
				
				for {set itrChannel 0} {$itrChannel < $channelGroup} {incr itrChannel} {
				    set dummyChannel [expr $itrGroup + ($itrChannel* ($channelNumber/$channelGroup) ) ]
				    set fileName "E.$itrSide.$itrAsic.$dummyChannel"
				    # Empty the content of the histogram 
				    # Check also that the histogram has been created.
				    if { [ZeroSpectrum $fileName] != 0 } { 
					return "Histograms E.$itrSide.$itrAsic.* were not created";}
				}
				
				# Wait.
				after $deltaT ;# [ms] 
				
				# Save data to file.
				for {set itrChannel 0} {$itrChannel < $channelGroup} {incr itrChannel} {
				    set dummyChannel [expr $itrGroup + ($itrChannel* ($channelNumber/$channelGroup) ) ]
				    set plot  [do_analyseSpectra $itrSide $itrAsic $dummyChannel]
				    set range [lindex $plot 0]
				    set data  [lindex $plot 1]
				    puts $fileInput "$dummyChannel $itrEth $range"
				    puts $fileInput $data
				    flush $fileInput
				}
				
				# Power off channel.
				for {set itrChannel 0} {$itrChannel < $channelGroup} {incr itrChannel} {
				    set dummyChannel [expr $itrGroup + ($itrChannel* ($channelNumber/$channelGroup) ) ]
				    set registerPwr [expr int ( 80 + ($dummyChannel/8) )]
				    WriteR3BASICRegister [expr ($itrSide << 18) + ($itrAsic << 10) + $registerPwr] 255
				}
				
			    } ;# End loop over channels
			    
			} ;# End loop over Eth.
			
			close $fileInput
			
			# --
			# TO BE REMOVED.
		    } ;# end of foreach itrPg1Offset
		} ;# end of foreach itrPkTm
		#--
		
	    } ;# end of loop over asic.
	}  ;#end of loop over side.
	
	# Stop DAQ
	do_getState
	after 500 set timeout 1
	do_stop
	vwait timeout
	# Mandatory to enquire state
	if { [do_getState] != 0 } { return "Daq did not stop correctly";}
	
	return "Measurement completed" 
    }
    
    proc  do_click {}  {
	variable cgidata
	variable started
	variable LogFlag
	variable ShowLogFlag
	
	set cgidata [ncgi::nvlist]
	
	if {[llength $cgidata] == 0} then {return}
	
	if {[llength $cgidata] < 2} then {
	    InsertLog "CGI data is too short: $cgidata"
	} else {
	    if {[lindex $cgidata 0] != "Widget"} then {
		InsertLog "Invalid CGI data: $cgidata"
	    } else {
		
		set w [first [lindex $cgidata 1]]
		
		switch -glob $w {
		    
		    INITIALIZE_ASICS { InsertLog "[do_powerOffFrontEnd]" }
		    INITIALIZE_DAQ   { InsertLog "[do_initializeDaq $cgidata]" }
		    LOAD_OFFSET      { InsertLog "[do_loadOffset $cgidata]" }
		    MEASURE_ETH      { InsertLog "[do_measureEth $cgidata]" }
		    NOISE_FLOOR      { InsertLog "[do_noiseFloor $cgidata]" } 
		    RESET            { set started 0; }
		    TUNE_ETH_COARSE  { InsertLog "[do_tuneEthCoarse $cgidata]"  }
		    TUNE_ETH_COARSE_BIS { InsertLog "[do_tuneEthCoarseBis $cgidata]"  }
		    TUNE_ETH_COARSE_TER { InsertLog "[do_tuneEthCoarseTer $cgidata]"  }
		    TUNE_ETH_FINE    { InsertLog "[do_tuneEthFine $cgidata]"  }
		    
		    default {InsertLog "unexpected CGI data:  $cgidata"}
		}
	    }
        }
	do_redisplay
    }
    
}   ;# end of if {$started == 0}

# ---------------------------
# # Loop over individual channels.
# set channelNumber 128
# set channelIncrease 1
# for {set itrChannel 0} {$itrChannel < $channelNumber} {set itrChannel [expr $itrChannel+$channelIncrease]} {

#     # if { $itrChannel >= 10 } { continue;}

#     set skipOffset 64
#     set rangeOffset 127
#     for {set itrOffset [expr 255-$skipOffset]} {$itrOffset > $rangeOffset} {set itrOffset [expr $itrOffset-4]} {

# 	for {set itrCh 0} {$itrCh < $channelIncrease} {incr itrCh} {
# 	    set dummyChannel [expr $itrChannel+$itrCh]
# 	    set page 3
# 	    WriteR3BASICRegister [expr ($itrSide << 18) + ($itrAsic << 10) + ($page << 8) + (128+$dummyChannel) ] $itrOffset  
# 	}

# 	## set valuePwr [expr 255 - pow (2, $itrChannel - ($itrChannel/8)*8 ) ]
# 	set registerPwr [expr 80 + ($itrChannel/8)]
# 	WriteR3BASICRegister [expr ($itrSide << 18) + ($itrAsic << 10) + $registerPwr] 0

# 	# Wait for power up.
# 	if {$itrOffset == [expr 255-$skipOffset]} {after 500}

# 	# Zero histo.
# 	for {set itrCh 0} {$itrCh < $channelIncrease} {incr itrCh} {
# 	    set dummyChannel [expr $itrChannel+$itrCh]
# 	    set fileName "E.$itrSide.$itrAsic.$dummyChannel"
# 	    # This zeros the histogram.
# 	    ZeroSpectrum $fileName
# 	}

# 	# Wait.
# 	after 100

# 	for {set itrCh 0} {$itrCh < $channelIncrease} {incr itrCh} {
# 	    set dummyChannel [expr $itrChannel+$itrCh]
# 	    set plot  [do_analyseSpectra $itrSide $itrAsic $dummyChannel]
# 	    set range [lindex $plot 0]
# 	    set data  [lindex $plot 1]

# 	    set offsetLabel [expr (127-$itrOffset)]
# 	    puts $fileInput "$itrSide $itrAsic $dummyChannel $offsetLabel $range"
# 	    # puts $fileInput "$itrSide $itrAsic $itrChannel $itrOffset $range"
# 	    puts $fileInput $data
# 	    flush $fileInput
# 	}

# 	# Power off channel.
# 	WriteR3BASICRegister [expr ($itrSide << 18) + ($itrAsic << 10) + $registerPwr] 255

#     } ;# end of loop over offset.

#     set rangeOffset 128
#     for {set itrOffset 0} {$itrOffset < [expr $rangeOffset-$skipOffset]} {set itrOffset [expr $itrOffset+4]} {

# 	for {set itrCh 0} {$itrCh < $channelIncrease} {incr itrCh} {
# 	    set dummyChannel [expr $itrChannel+$itrCh]
# 	    set page 3
# 	    WriteR3BASICRegister [expr ($itrSide << 18) + ($itrAsic << 10) + ($page << 8) + (128+$dummyChannel) ] $itrOffset 
# 	}
# 	# set valuePwr [expr 255 - pow (2, $itrChannel - ($itrChannel/8)*8 ) ]
# 	set registerPwr [expr 80 + ($itrChannel/8)]
# 	WriteR3BASICRegister [expr ($itrSide << 18) + ($itrAsic << 10) + $registerPwr] 0 ;#$valuePwr
# 	# Wait for power up.
#      	# after 2

# 	# Zero histo.
# 	for {set itrCh 0} {$itrCh < $channelIncrease} {incr itrCh} {
# 	    set dummyChannel [expr $itrChannel+$itrCh]
# 	    set fileName "E.$itrSide.$itrAsic.$dummyChannel"
# 	    # This should zero the histogram.
# 	    ZeroSpectrum $fileName
# 	}
# 	# Wait.
# 	after 100

# 	for {set itrCh 0} {$itrCh < $channelIncrease} {incr itrCh} {
# 	    set dummyChannel [expr $itrChannel+$itrCh]
# 	    set plot  [do_analyseSpectra $itrSide $itrAsic $dummyChannel]
# 	    set range [lindex $plot 0]
# 	    set data  [lindex $plot 1]

# 	    puts $fileInput "$itrSide $itrAsic $dummyChannel $itrOffset $range"
# 	    puts $fileInput $data
# 	    flush $fileInput
# 	}

# 	# Power off channel.
#      	WriteR3BASICRegister [expr ($itrSide << 18) + ($itrAsic << 10) + $registerPwr] 255
#     } ;# end loop over offset

# }  ;# end of loop over channel.
# -----------------------------    