

#    experiment run control for R3B - adapted from AIDA

global MIDAS_DataAcquisitionControlServer_Verbose
set MIDAS_DataAcquisitionControlServer_Verbose 0

global Options

global NumASICs NumChans
set NumASICs 32
set NumChans 128


global R3BASICBase
set R3BASICBase 16

    variable SIDEList
    if {![info exists SIDEList]} {set SIDEList {0}}
    variable ASICList0 {0}
    if {![info exists ASICList0]} {set ASICList0 {0}}
    variable ASICList1 {0}
    if {![info exists ASICList1]} {set ASICList1 {0}}


variable SpecBase [file join $env(MIDASBASE) SpecData R3B]
variable SpecPath

proc Dump_Raw_Data {} {return "[Dump_Raw_R3B_Data]\n"}

proc Set_Options {} {
   global Options

    LoadOptions

    if {![info exists Options(Hist_MaxChans)]} {set Options(Hist_MaxChans) [expr 8*1024*1024]}

    if {![info exists Options(R3B_GroupBase)]} {set Options(R3B_GroupBase) 1}
    if {![info exists Options(Include.R3B)]}   {set Options(Include.R3B) 1}
    
    if {![info exists Options(Rate.channels)]} {set Options(Rate.channels) [expr 128*32*2]}
    if {![info exists Options(Stat.channels)]} {set Options(Stat.channels) [expr 128*32*2]}
    if {![info exists Options(Stat.shift)]}    {set Options(Stat.shift) 0}
    if {![info exists Options(Stat.offset)]}   {set Options(Stat.offset) 0}
    
    if {![info exists Options(R3B.channels)]} {set Options(R3B.channels) 4096}
    if {![info exists Options(R3B.shift)]}    {set Options(R3B.shift) 0}
    if {![info exists Options(R3B.offset)]}   {set Options(R3B.offset) 0}
    
    if {![info exists Options(R3B_Hist_E_Enable)]} {set Options(R3B_Hist_E_Enable) 1}
    if {![info exists Options(R3B_Hist_Hit0_Enable)]} {set Options(R3B_Hist_Hit0_Enable) 0}
    if {![info exists Options(R3B_Hist_Hit1_Enable)]} {set Options(R3B_Hist_Hit1_Enable) 1}
    if {![info exists Options(R3B_Hist_Hit01_Enable)]} {set Options(R3B_Hist_Hit01_Enable) 0}
    if {![info exists Options(R3B_Hist_DT_Enable)]} {set Options(R3B_Hist_DT_Enable) 0}

    if {![info exists Options(R3B.DTchannels)]} {set Options(R3B.DTchannels) 4000}
    if {![info exists Options(R3B.DTshift)]}    {set Options(R3B.DTshift) 4}
    if {![info exists Options(R3B.DToffset)]}   {set Options(R3B.DToffset) 0}
    
    if {![info exists Options(R3B_ValPulseWidth)]} {set Options(R3B_ValPulseWidth) 800}
    if {![info exists Options(R3B_ValPulseDelay)]} {set Options(R3B_ValPulseDelay) 19200}
    if {![info exists Options(R3B_ResetPulseWidth)]} {set Options(R3B_ResetPulseWidth) 2000}

    for {set i 0} {$i < 2} {incr i} {
        for {set j 0} {$j < 16} {incr j} {
           if {![info exists Options(R3B_Hist_[set i].[set j]_Enable)]} {set Options(R3B_Hist_[set i].[set j]_Enable) 1}
        }
    }


    
    SaveOptions
    
}

proc Dump_Raw_R3B_Data {} {
    
    set info "data $::DataAcquisition::ACQSERVER\n"
    
    for {set offset 0} {$offset <= 500} {incr offset 100} {
	
	DataAcqCommand 19 "Getting Raw Data" "19 $offset" 2
	set data [ReadNetVar NetVar.EXEC.Response]
	
	set c 0
	
	for {set i 0} {$i < [llength $data]} {incr i} {
	    set word [lindex $data $i]
	    if {$word == 0xaa5555aa} then {
		append info "\n$word - end of dump\n"
		break
	    }
	    if {$word == 0x55aaaa55} then {
		append info "\n$word - end of event\n"
		continue
	    }
	    if {[expr ($word >> 30) & 3] == 3} {
		incr i
		set t [lindex $data $i]
		set hit [expr ($word >> 29) & 1]
		set chan [expr ($word >> 12) & 127]
		set asic [expr ($word >> 19) & 15]
		set id [expr ($word >> 23) & 0x3f]
		set adc [format "0x%x" [expr $word & 0xfff]]
		append info "\nData Item $word $t; Hit=$hit; ID=$id; ASIC=$asic; Channel=$chan; ADC=$adc; Time=$t"
	    }
	    
	    if {[expr ($word >> 30) & 3] == 2} {
	      if {[expr ($word >> 20) & 15] == 4 || [expr ($word >> 20) & 15] == 5} {
		incr i
		set t [lindex $data $i]
		set time [format "0x%x" [expr $word & 0x000fffff]]
		append info "\nWR Time Item $word $t; Time (high)=$time; Time (low)=$t"
              }
	      if {[expr ($word >> 20) & 15] == 7 || [expr ($word >> 20) & 15] == 8} {
		incr i
		set t [lindex $data $i]
		set time [format "0x%x" [expr $word & 0x000fffff]]
		append info "\nR3B Time Item $word $t; Time (high)=$time; Time (low)=$t"
              }
	      if {[expr ($word >> 20) & 15] == 14 || [expr ($word >> 20) & 15] == 15} {
		incr i
		set t [lindex $data $i]
		set time [format "0x%x" [expr $word & 0x000fffff]]
		append info "\nR3B Ext Event $word $t; Time (high)=$time; Time (low)=$t"
              }
	    }

	    if {[expr ($word >> 30) & 3] == 0} {
		append info "\nDebug Item $word"
	    }
	    
	    incr c
	    
	    if {$c == 10} then {append info "\n"; set c 0}
	}
    }
    
    return "$info"
}

proc create_R3B_spectra {} {
    global Options
    global NumChans
       variable SIDEList
       variable ASICList0
       variable ASICList1
    
    set Me $::DataAcquisition::ACQSERVER

    set Chs 0
    
    foreach SIDE $SIDEList {
	foreach ASIC [set ASICList$SIDE] {

        if {$Options(R3B_Hist_[set SIDE].[set ASIC]_Enable) == 1} {

           if {$Chs > $Options(Hist_MaxChans)} {puts "Too many histogram channels requested ($Chs > $Options(Hist_MaxChans))"; return}
	    
	    puts "Creating spectra for SIDE $SIDE ASIC $ASIC"
	    
	    #    creating spectra for ADC data
	    
	    for {set Ch 0} {$Ch < $NumChans} {incr Ch} {
		
		if {$Options(R3B_Hist_E_Enable) == 1} {
		    
		    set z [catch {CreateSpectrum1D E.$SIDE.$ASIC.$Ch $Options(R3B.channels)} m]
		    
		    #    Allow "Spectrum already exists" so that further spectra can be created
		    if {$z!=0 && $m==0x30004} then {set z 0}
		    if {$z!=0} then {
			puts "Error attempting to create spectrum E.$SIDE.$ASIC.$Ch"
			error $m
		    }
		    
		    set z [catch {SpectrumPutTitle E.$SIDE.$ASIC.$Ch [set Me]_E.$SIDE.$ASIC.$Ch} m]

                    incr Chs $Options(R3B.channels)
		    
		}
	    }

	    #    creating spectra for time difference
	    
	    for {set Ch 0} {$Ch < $NumChans} {incr Ch} {
		
		if {$Options(R3B_Hist_DT_Enable) == 1} {
		    
		    set z [catch {CreateSpectrum1D DT.$SIDE.$ASIC.$Ch $Options(R3B.DTchannels)} m]
		    
		    #    Allow "Spectrum already exists" so that further spectra can be created
		    if {$z!=0 && $m==0x30004} then {set z 0}
		    if {$z!=0} then {
			puts "Error attempting to create spectrum DT.$SIDE.$ASIC.$Ch"
			error $m
		    }
		    
		    set z [catch {SpectrumPutTitle DT.$SIDE.$ASIC.$Ch [set Me]_DT.$SIDE.$ASIC.$Ch} m]

                    incr Chs $Options(R3B.DTchannels)
		    
		}
	    }
	} }
    }
}

proc map_R3B_spectra {} {
    upvar #0 SpectraHandle f
    global Options
    global NumChans
       variable SIDEList
       variable ASICList0
       variable ASICList1
    
    set Me $::DataAcquisition::ACQSERVER
    
    set SpectraDefs [file join /tmp SpectraDefs.dat]
    
    set z [catch {open $SpectraDefs w} f]
    if {$z != 0} then {
	puts "Cannot open file $SpectraDefs $f"
	return
    }
    
    map_spectrum "Stat 0 0 $Options(Stat.offset) $Options(Stat.shift)" 

   set Chs 0
    
    foreach SIDE $SIDEList {
	foreach ASIC [set ASICList$SIDE] {

        if {$Options(R3B_Hist_[set SIDE].[set ASIC]_Enable) == 1} {

           if {$Chs > $Options(Hist_MaxChans)} {continue}
	    
	    puts "Mapping spectra for  SIDE $SIDE ASIC $ASIC"
	    
	    for {set Ch 0} {$Ch < $NumChans} {incr Ch} {
		
		if {$Options(R3B_Hist_E_Enable) == 1} {

                   incr Chs $Options(R3B.channels)

		   if {$Options(R3B_Hist_Hit0_Enable) == 1} {
		    
		       set group [expr $Options(R3B_GroupBase)+32+($SIDE*16)+$ASIC]
		       set item $Ch
		       map_spectrum "E.$SIDE.$ASIC.$Ch $group $item $Options(R3B.offset) $Options(R3B.shift)"
  
                   }
		   if {$Options(R3B_Hist_Hit1_Enable) == 1} {
		    
		       set group [expr $Options(R3B_GroupBase)+($SIDE*16)+$ASIC]
		       set item $Ch
		       map_spectrum "E.$SIDE.$ASIC.$Ch $group $item $Options(R3B.offset) $Options(R3B.shift)"
  
                   }
		   if {$Options(R3B_Hist_Hit01_Enable) == 1} {
		    
		       set group [expr $Options(R3B_GroupBase)+64+($SIDE*16)+$ASIC]
		       set item $Ch
		       map_spectrum "E.$SIDE.$ASIC.$Ch $group $item $Options(R3B.offset) $Options(R3B.shift)"
                   }
		}

	    }

	    for {set Ch 0} {$Ch < $NumChans} {incr Ch} {
		
		if {$Options(R3B_Hist_DT_Enable) == 1} {
		    
		    set group [expr $Options(R3B_GroupBase)+96+($SIDE*16)+$ASIC]
		    set item $Ch
		    map_spectrum "DT.$SIDE.$ASIC.$Ch $group $item $Options(R3B.DToffset) $Options(R3B.DTshift)"

                    incr Chs $Options(R3B.DTchannels)
		}

	    }
	} }
    }
    
    close $f
}

proc doSetUp {} {
    global Options
       variable SIDEList
       variable ASICList0
       variable ASICList1
    
    puts "executing doSetUp"
    
    Set_Options; make-UIregisters

    set Me $::DataAcquisition::ACQSERVER

    probe_i2c

  puts "doSetUp: $SIDEList &&   $ASICList0  &&   $ASICList1"
    
    DataAcqCommand 1 "Halt"
    
    unmap_spectra
    delete_spectra
    
    CreateSpectrum1D Stat $Options(Stat.channels)
    set z [catch {SpectrumPutTitle Stat [set Me]_Stat} m]
    
    CreateSpectrum1D Rate $Options(Rate.channels)
    set z [catch {SpectrumPutTitle Rate [set Me]_Rate} m]
    
    create_R3B_spectra
    
    map_R3B_spectra 
    
    map_spectra

    ParameterRestore $Options(ASIC.settings) -1
    
    Setup_Electronics   ;# do AFTER restore which allows force of certain settings - NOTE finishes with an ASIC Reset
    
    DataAcqCommand 3 "Setup"
    
    WriteNetVar NetVar.DACQ.PollTicks 100
    WriteNetVar NetVar.DACQ.Crdoutenable 1   ;# enable data readout
    
    puts "completed doSetUp"

}

   proc ParameterRestore {DBkey p} {
      global NumASICs
       variable SIDEList
       variable ASICList0
       variable ASICList1

      set ::R3B::DBNodeBase $DBkey
      set ::R3B::DBNodeName $::DataAcquisition::ACQSERVER
      
      puts "Restoring settings from $::R3B::DBRootName/$::R3B::DBNodeBase for $::DataAcquisition::ACQSERVER"

#    first restore ASIC settings

   foreach side $SIDEList {
     foreach asic [set ASICList$side] {

         set ASIC [expr ($side*16) + $asic]

         if {[expr ($p >> [expr $NumASICs-$ASIC-1]) & 1] == 1} {
         
            for {set i 1} {$i <= 5} {incr i} {    ;# avoid REG0
              set vl [DBGetValue ASIC.$ASIC.$i]
              if {$i == 1} {set vl [expr ($vl & 0xc0) + 0x30]}
              WriteR3BASICRegister [expr ($side << 18) + ($asic << 10) + $i] $vl
            }

            for {set i 10} {$i <= 21} {incr i} {
              set vl [DBGetValue ASIC.$ASIC.$i]
              WriteR3BASICRegister [expr ($side << 18) + ($asic << 10) + $i] $vl
            }

#           for {set i 64} {$i <= 79} {incr i} {
#             set vl [DBGetValue ASIC.$ASIC.$i]
#             WriteR3BASICRegister [expr ($side << 18) + ($asic << 10) + $i] $vl
#           }

#           for {set i 80} {$i <= 95} {incr i} {
#             set vl [DBGetValue ASIC.$ASIC.$i]
#             WriteR3BASICRegister [expr ($side << 18) + ($asic << 10) + $i] $vl
#           }
         }

      }
   }

      puts "Settings restored from $::R3B::DBRootName/$::R3B::DBNodeBase for $::DataAcquisition::ACQSERVER"


#       ::R3B::WriteRegister HasBeenRestored 1

#        Elog INFORMATION "FEE64 settings restored from $::R3B::DBRootName/$::R3B::DBNodeBase for $::DataAcquisition::ACQSERVER"

   }

   proc ParameterSave {DBkey p} {
      global NumASICs
       variable SIDEList
       variable ASICList0
       variable ASICList1

      set ::R3B::DBNodeBase $DBkey
      set ::R3B::DBNodeName $::DataAcquisition::ACQSERVER
      
      puts "[date +%Y%h%d-%H.%M.%S] Saving settings ($p) to $::R3B::DBRootName/$::R3B::DBNodeBase for $::DataAcquisition::ACQSERVER"

#    first save ASIC settings

   foreach side $SIDEList {
     foreach asic [set ASICList$side] {

         set ASIC [expr ($side*16) + $asic]

         if {[expr ($p >> [expr $NumASICs-$ASIC-1]) & 1] == 1} {

            for {set i 1} {$i <= 5} {incr i} {   ;# avoid REG0
              set vl [ReadR3BASICRegister [expr ($side << 18) + ($asic << 10) + $i]]
              DBSetValue ASIC.$ASIC.$i [format "0x%02x" [expr $vl & 255]]
            }

            for {set i 10} {$i <= 21} {incr i} {
              set vl [ReadR3BASICRegister [expr ($side << 18) + ($asic << 10) + $i]]
              DBSetValue ASIC.$ASIC.$i [format "0x%02x" [expr $vl & 255]]
            }

#           for {set i 64} {$i <= 79} {incr i} {
#             set vl [ReadR3BASICRegister [expr ($side << 18) + ($asic << 10) + $i]]
#             DBSetValue ASIC.$ASIC.$i [format "0x%02x" [expr $vl & 255]]
#           }

#           for {set i 80} {$i <= 95} {incr i} {
#             set vl [ReadR3BASICRegister [expr ($side << 18) + ($asic << 10) + $i]]
#             DBSetValue ASIC.$ASIC.$i [format "0x%02x" [expr $vl & 255]]
#           }
         }
      }
   }

      puts "[date +%Y%h%d-%H.%M.%S] Settings saved to $::R3B::DBRootName/$::R3B::DBNodeBase for $::DataAcquisition::ACQSERVER"

#       ::R3B::WriteRegister HasBeenRestored 1

#        Elog INFORMATION "R3B4K settings saved to $::R3B::DBRootName/$::R3B::DBNodeBase for $::DataAcquisition::ACQSERVER"

   }

proc Reset_Electronics {} {    ;#   used by RESET

    set Me $::DataAcquisition::ACQSERVER

     WriteR3BRegister 0x100 2        ;#  set reset DMA
     WriteR3BRegister 0x100 0        ;#  and clear

     WriteR3BRegister 0x106 0        ;#  and clear rd_ptr

}

proc Setup_Asics {} {
       variable SIDEList
       variable ASICList0
       variable ASICList1

#  Val input not active (permanently enabled)
#     side=0 electrons mode (N);  side=1 holes mode (P)

#    note clocks now 200 MHz

    foreach side $SIDEList {
    if {$side == 0} {set charge 1} else {set charge 0}
    foreach asic [set ASICList$side] {

       set i 1
       WriteR3BASICRegister [expr ($side << 18) + ($asic << 10) + $i]  [expr 0x30 + ($charge << 7)]
       switch $asic {
           0 -
           1 -
           2 -
           3 {
               set i 2
               WriteR3BASICRegister [expr ($side << 18) + ($asic << 10) + $i]  0x24
           }

           12 -
           13 -
           14 -
           15 {
               set i 2
               WriteR3BASICRegister [expr ($side << 18) + ($asic << 10) + $i]  0x54
           }

           default {
               set i 2
               WriteR3BASICRegister [expr ($side << 18) + ($asic << 10) + $i] 0x04
           }
        }
     }
     }
}

proc Setup_Electronics {} {    ;#   used by SETUP

   puts "starting Setup_Electronics"

    set Me $::DataAcquisition::ACQSERVER

     WriteR3BRegister 0x100 2        ;#  set reset DMA
     WriteR3BRegister 0x100 0        ;#  and clear

     WriteR3BRegister 0x102 0x0f000000   ;# set DMA MEM Base
     WriteR3BRegister 0x103 2048         ;# set DMA burst length
     WriteR3BRegister 0x105 8192  ;# set DMA MEM Length (units of burst length) (0x01000000/2048) == (16M/2K)
     WriteR3BRegister 0x104 256         ;# set DMA Buffer Length (units of burst length) (32 buffers)  (8192/32)

     WriteR3BRegister 0x106 0        ;#  and clear rd_ptr

    Setup_Asics   ;#  put R2 to default settings

    do_ResetTimeStamp

    do_ASICReset

    do_Calibrate

     setModID

   puts "finishing Setup_Electronics"

}

proc Stop_Electronics {} {

     WriteR3BRegister 0x100 0        ;#    unset enable DMA

}

proc Go_Electronics {} {

    set Me $::DataAcquisition::ACQSERVER

     WriteR3BRegister 0x100 2        ;#  set reset DMA
     WriteR3BRegister 0x100 0        ;#  and clear

     WriteR3BRegister 0x106 0        ;#  and clear rd_ptr

     WriteR3BRegister 0x100 1        ;#  enable  DMA

}

    proc HistogramZeroAll {args} {

       set SpecNames [SpectrumNames]
       set NumSpec 0

       ZeroSpectrum Stat
       incr NumSpec

      foreach item $SpecNames {
         if {[first $item] != 1} {continue}   ;#   not a 1D histogram !!

         set SpecName [second $item]

         ZeroSpectrum $SpecName
         incr NumSpec
      }

       puts "$NumSpec spectra zeroed"
    }

    proc HistogramZero {p} {   ;#  supports Zero for histograms in selected ASICs
      global NumASICs
       variable SIDEList
       variable ASICList0
       variable ASICList1

       set SpecNames [SpectrumNames]
       set NumSpec 0

       ZeroSpectrum Stat
       incr NumSpec

   foreach side $SIDEList {
     foreach asic [set ASICList$side] {

         set ASIC [expr ($side*16) + $asic]

         if {[expr ($p >> [expr $NumASICs-$ASIC-1]) & 1] == 1} {

            foreach item $SpecNames {
               if {[first $item] != 1} {continue}   ;#   not a 1D histogram !!

               set SpecName [second $item]

               if {$side != [second [split $SpecName .]]} {continue}
               if {$asic != [third [split $SpecName .]]} {continue}

               ZeroSpectrum $SpecName
               incr NumSpec
            }
         }

      }
   }

       puts "$NumSpec spectra zeroed"
    }


 proc SpecSave1D {spec base range type title counts} {
    variable SpecPath
     
     set f3 [file join $SpecPath $spec.msf]
     set z1 [catch {open $f3 w} m1]
     set z2 [catch {file attributes $f3 -permissions rwxrwxrwx} m2]

      if {$z1 != 0} then {
          puts "Cannot create $f3 - $m1"
      }
      if {$z2 != 0} then {
          puts "error accessing $f3 - $m2"
      }

      set f $m1

      fconfigure $f -translation binary -encoding binary

  set ctime [clock format [clock seconds] -format {%d-%b-%Y %H:%M:%S}]
  set mtime [clock format [clock seconds] -format {%d-%b-%Y %H:%M:%S}]

      puts -nonewline $f [binary format i1 412900921]   ;#   magic number
      puts -nonewline $f [binary format i1 1]           ;#   version
      puts -nonewline $f [binary format a32 $spec]      ;#   spectrum name
      puts -nonewline $f [binary format i1 1]           ;#   dimension
      puts -nonewline $f [binary format a20 $ctime]     ;#   creation time/date
      puts -nonewline $f [binary format a20 $mtime]     ;#   modification time/date
      puts -nonewline $f [binary format i1 $base]       ;#   base0

      for {set i 2} {$i <=8} {incr i} {
         puts -nonewline $f [binary format i1 -1]     ;#   baseI
      }

      puts -nonewline $f [binary format i1 $range]    ;#   range0

      for {set i 2} {$i <=8} {incr i} {
         puts -nonewline $f [binary format i1 -1]     ;#   rangeI
      }

      puts -nonewline $f [binary format i1 436]        ;#   spectrum title offset  (was -1)

      for {set i 2} {$i <=32} {incr i} {
         puts -nonewline $f [binary format i1 -1]     ;#  information pointers
      }

      for {set i 1} {$i <=8} {incr i} {
         puts -nonewline $f [binary format i1 -1]     ;#  annotation pointers
      }

      for {set i 1} {$i <=8} {incr i} {
         puts -nonewline $f [binary format i1 -1]     ;#  calibration pointers
      }

      for {set i 1} {$i <=8} {incr i} {
         puts -nonewline $f [binary format i1 -1]     ;#  efficiency pointers
      }

                                                     ;#  data array 1 descriptor 
      puts -nonewline $f [binary format i1 0]        ;#  array layout
      puts -nonewline $f [binary format i1 $type]    ;#  array type  (32 bit unsigned)
      puts -nonewline $f [binary format i1 0]        ;#  reserved 
      puts -nonewline $f [binary format i1 0]        ;#  reserved
      puts -nonewline $f [binary format i1 0]        ;#  spectrum counts offset

                                                     ;#  data array 2 descriptor 
      puts -nonewline $f [binary format i1 0]        ;#  array layout
      puts -nonewline $f [binary format i1 $type]    ;#  array type 
      puts -nonewline $f [binary format i1 0]        ;#  reserved 
      puts -nonewline $f [binary format i1 0]        ;#  reserved
      puts -nonewline $f [binary format i1 -1]       ;#  spectrum counts offset

      puts -nonewline $f [binary format i1 436]      ;#  base of string space
      puts -nonewline $f [binary format i1 20]       ;#  string free space
      puts -nonewline $f [binary format i1 76]       ;#  top of string space

      puts -nonewline $f [binary format i1 512]      ;#  base of counts space
      puts -nonewline $f [binary format i1 $range]   ;#  counts free space
      puts -nonewline $f [binary format i1 $range]   ;#  top of counts space
 
      puts -nonewline $f [binary format a20 $title]  ;# string space - title

      for {set i 456} {$i < 512} {incr i} {          ;#
         puts -nonewline $f [binary format c1 0]     ;#  unused
      }

      puts -nonewline $f [binary format i* $counts]  ;# counts

      close $f
}

    proc HistogramSaveAll {DBkey p} {
       variable SIDEList
       variable ASICList0
       variable ASICList1
       variable SpecPath
       global env
       global NumASICs

      set f1 $DBkey
      set z1 [catch {file mkdir $f1} m1]
      set z2 [catch {file attributes $f1 -permissions rwxrwxrwx} m2]

      if {$z1 != 0} then {
          puts "Cannot create $f1 - $m1"
      }
      if {$z2 != 0} then {
          puts "error accessing $f1 - $m2"
      }

      set f1 [file join $f1 $env(ComputerName)]
      set z1 [catch {file mkdir $f1} m1]
      set z2 [catch {file attributes $f1 -permissions rwxrwxrwx} m2]

      if {$z1 != 0} then {
          puts "Cannot create $f1 - $m1"
      }
      if {$z2 != 0} then {
          puts "error accessing $f1 - $m2"
      }

      set f2 [file join $f1 [date +%Y%h%d-%H.%M.%S]]
      set z1 [catch {file mkdir $f2} m1]
      set z2 [catch {file attributes $f2 -permissions rwxrwxrwx} m2]

      if {$z1 != 0} then {
          puts "Cannot create $f2 - $m1"
      }
      if {$z2 != 0} then {
          puts "error accessing $f2 - $m2"
      }

      puts "[date +%Y%h%d-%H.%M.%S] Saving Histograms to $f2 for $::DataAcquisition::ACQSERVER"

       set SpecPath $f2

       set SpecNames [SpectrumNames]
       set NumSpec 0

         set SpecName Stat

         set info [SpectrumDetails $SpecName]
         set channels [lindex $info 2]

         set title [SpectrumGetTitle $SpecName]
         set counts [SpectrumRead1D $SpecName 0 $channels]

         set z [catch {SpecSave1D $SpecName 0 $channels 5 $title $counts} m]
         if {$z!=0} then {
             puts "Error attempting to save spectrum $SpecName: $m"
         } else {
             incr NumSpec
         }

       foreach item $SpecNames {
         if {[first $item] != 1} {continue}    ;#  not a 1D histogram

         set SpecName [second $item]

         set info [SpectrumDetails $SpecName]
         set channels [lindex $info 2]

         set title [SpectrumGetTitle $SpecName]
         set counts [SpectrumRead1D $SpecName 0 $channels]

         set z [catch {SpecSave1D $SpecName 0 $channels 5 $title $counts} m]
         if {$z!=0} then {
             puts "Error attempting to save spectrum $SpecName: $m"
         } else {
             incr NumSpec
         }
       }

      puts "[date +%Y%h%d-%H.%M.%S] $NumSpec Histograms saved for $::DataAcquisition::ACQSERVER"

    }

    proc HistogramSave {DBkey p} {
       variable SIDEList
       variable ASICList0
       variable ASICList1
       variable SpecPath
       global env
       global NumASICs

      set f1 $DBkey
      set z1 [catch {file mkdir $f1} m1]
      set z2 [catch {file attributes $f1 -permissions rwxrwxrwx} m2]

      if {$z1 != 0} then {
          puts "Cannot create $f1 - $m1"
      }
      if {$z2 != 0} then {
          puts "error accessing $f1 - $m2"
      }

      set f1 [file join $f1 $env(ComputerName)]
      set z1 [catch {file mkdir $f1} m1]
      set z2 [catch {file attributes $f1 -permissions rwxrwxrwx} m2]

      if {$z1 != 0} then {
          puts "Cannot create $f1 - $m1"
      }
      if {$z2 != 0} then {
          puts "error accessing $f1 - $m2"
      }

      set f2 [file join $f1 [date +%Y%h%d-%H.%M.%S]]
      set z1 [catch {file mkdir $f2} m1]
      set z2 [catch {file attributes $f2 -permissions rwxrwxrwx} m2]

      if {$z1 != 0} then {
          puts "Cannot create $f2 - $m1"
      }
      if {$z2 != 0} then {
          puts "error accessing $f2 - $m2"
      }

      puts "[date +%Y%h%d-%H.%M.%S] Saving Histograms to $f2 for $::DataAcquisition::ACQSERVER with selection $p"

       set SpecPath $f2

       set SpecNames [SpectrumNames]
       set NumSpec 0

         set SpecName Stat

         set info [SpectrumDetails $SpecName]
         set channels [lindex $info 2]

         set title [SpectrumGetTitle $SpecName]
         set counts [SpectrumRead1D $SpecName 0 $channels]

         set z [catch {SpecSave1D $SpecName 0 $channels 5 $title $counts} m]
         if {$z!=0} then {
             puts "Error attempting to save spectrum $SpecName: $m"
         } else {
             incr NumSpec
         }

   foreach side $SIDEList {
     foreach asic [set ASICList$side] {

         set ASIC [expr ($side*16) + $asic]

         if {[expr ($p >> [expr $NumASICs-$ASIC-1]) & 1] == 1} {

       foreach item $SpecNames {
         if {[first $item] != 1} {continue}    ;#  not a 1D histogram

         set SpecName [second $item]

         if {$side != [second [split $SpecName .]]} {continue}
         if {$asic != [third [split $SpecName .]]} {continue}

         set info [SpectrumDetails $SpecName]
         set channels [lindex $info 2]

         set title [SpectrumGetTitle $SpecName]
         set counts [SpectrumRead1D $SpecName 0 $channels]

         set z [catch {SpecSave1D $SpecName 0 $channels 5 $title $counts} m]
         if {$z!=0} then {
             puts "Error attempting to save spectrum $SpecName: $m"
         } else {
             incr NumSpec
         }

       }
      }
     }
   }

      puts "[date +%Y%h%d-%H.%M.%S] $NumSpec Histograms saved for $::DataAcquisition::ACQSERVER"

    }


    proc TerminateExec {} {

        unmap_spectra
        DataAcqCommand 22 "Terminating DataAcq program" 0 2

    }

    proc LoadExec {} {

       set z [catch {eval exec nice [file join /MIDAS Data_Acq bin linux-ppc_4xx R3BExecV2.1] &} m]
       if {$z != 0} {puts "RunControl LoadExec returned z=$z and $m"}

    }

   proc do_ASICReset {} {
       global Options

      if {![info exists Options(R3B_ResetPulseWidth)]} {set Options(R3B_ResetPulseWidth) 2000}

       WriteR3BRegister 0x304 $Options(R3B_ResetPulseWidth)
       WriteR3BRegister 0x300 0x14   ;#  reset to load values
       WriteR3BRegister 0x300 0

       WriteR3BRegister 0x300 8; WriteR3BRegister 0x300 0;#  generate reset pulse
       after 10
   }

   proc do_Calibrate {} {
       variable SIDEList
       variable ASICList0
       variable ASICList1

       puts "running calibrate"

          foreach side $SIDEList {
            foreach asic [set ASICList$side] {

              set i 1    ;#  REG1
              set REG1.$side.$asic [ReadR3BASICRegister [expr ($side << 18) + ($asic << 10) + $i]]
              set vl [expr ([set REG1.$side.$asic] & 0xc0) + 0x00] 
              WriteR3BASICRegister [expr ($side << 18) + ($asic << 10) + $i] $vl

            }
          }
   
       do_calibration

          foreach side $SIDEList {
            foreach asic [set ASICList$side] {

              set i 1    ;#  REG1 
              WriteR3BASICRegister [expr ($side << 18) + ($asic << 10) + $i] [set REG1.$side.$asic]

            }
          }

       puts "finished calibrate"
   }

   proc setModID {} {

#    read Reg6 from one of the ASICs to obtain the ID picked up from the hardware
#    use this to set a netvar which is available to the data acq software at GO 

       variable SIDEList
       variable ASICList0
       variable ASICList1

       set side [first $SIDEList]
       set asic [first [set ASICList$side]]
       set i 6
       set ModID [expr [ReadR3BASICRegister [expr ($side << 18) + ($asic << 10) + $i]] & 0x3f]
       WriteNetVar NetVar.R3B.ModID $ModID

     for {set i 0} {$i <= 9} {incr i} {
        set Opt800.$i [ReadR3BRegister [expr 0x200 + $i]]
     }

   }

proc do_calibration {} {

    WriteR3BRegister [expr 0x200] 2   ;#  reset valid/invalid packet counters 
    after 100
    WriteR3BRegister [expr 0x200] 0
    
    WriteR3BRegister [expr 0x200] 1   ;#  start
    after 100
    WriteR3BRegister [expr 0x200] 0   ;#  stop

     for {set i 0} {$i <= 9} {incr i} {
        set Opt800.$i [ReadR3BRegister [expr 0x200 + $i]]
     }

     puts "Calibration: Status = [set Opt800.1]"
     for {set i 0} {$i < 8} {incr i} {
         set j [expr $i + 2]
         puts "Calibration: Chain = $i Status = [set Opt800.$j]"
     }
    
}
   
   proc do_PowerAll {vl} {      ;#   note - this copied from SETUP
       variable SIDEList
       variable ASICList0
       variable ASICList1

          foreach side $SIDEList {
             foreach asic [set ASICList$side] {
          
                 for {set i 80} {$i <= 95} {incr i} {
                    WriteR3BASICRegister [expr ($side << 18) + ($asic << 10) + $i] $vl
                 }
             }
          }
   }
 
   proc do_ResetTimeStamp {} {
       variable SIDEList
       variable ASICList0
       variable ASICList1

          foreach side $SIDEList {
             foreach asic [set ASICList$side] {

                 set i 1    ;#  REG1
                 set REG1.$side.$asic [ReadR3BASICRegister [expr ($side << 18) + ($asic << 10) + $i]]
                 set vl [expr ([set REG1.$side.$asic] & 0xc0) + 0x20]  ;# VAL input resets the counter
                 WriteR3BASICRegister [expr ($side << 18) + ($asic << 10) + $i] $vl

             }
          }

          WriteR3BRegister 0x400 2   ;#  asic_val => ts_fpga_reset

          after 1000   ;#  allow 1 sec delay

          WriteR3BRegister 0x400 0   ;#  asic_val NOT => ts_fpga_reset

          foreach side $SIDEList {
             foreach asic [set ASICList$side] {

                 set i 1    ;#  REG1 
                 WriteR3BASICRegister [expr ($side << 18) + ($asic << 10) + $i] [set REG1.$side.$asic]
             }
          }

          puts "Time Stamp Reset complete"

   }

   proc probe_i2c {} {
       variable SIDEList
       variable ASICList0
       variable ASICList1

       puts "Checking I2C buses"

          set SIDEList {}

          foreach SIDE {0 1} {

             set ASICList$SIDE {}

             foreach ASIC {0 1 2 3 4 5 6 7 8 9 10 11 12 13 14 15} {
                set i 0
                set z [catch {WriteR3BASICRegister [expr ($SIDE << 18) + ($ASIC << 10) + $i] 0xff} m]
                if {$z != 0} {puts "SIDE $SIDE; ASIC $ASIC; Write register $i 0xff returned $m"}
                set z [catch {ReadR3BASICRegister [expr ($SIDE << 18) + ($ASIC << 10) + $i]} m]
                if {$z != 0} {puts "SIDE $SIDE; ASIC $ASIC; Read register $i returned $m"}
                set vl $m
                if {$vl == 0xff} {lappend ASICList$SIDE $ASIC}
                WriteR3BASICRegister [expr ($SIDE << 18) + ($ASIC << 10) + $i] 0x0
             }
 
             if {[set ASICList$SIDE] != {}} {lappend SIDEList $SIDE}
          }

        puts "I2C bus check complete"

          foreach SIDE {0 1} {
             if {[set ASICList$SIDE] == {}} {
                puts "SIDE $SIDE not present or empty"
             } else {
                puts "SIDE $SIDE: ASICs discovered [set ASICList$SIDE]"
             }
          }
   }

 
    variable registerlist
    lappend registerlist [list NetVar.R3B.ModID  "-n R3B_ModID -t int"]
    lappend registerlist [list NetVar.R3B.ModuleNum  "-n R3B_ModuleNum -t int"]

    make-UIregisters

    Set_Options
  

