


variable OutputClockEnable 0
variable ClockDelayEnable 0
variable ClockDelay 0


    proc GetPLLConfig {} {
        variable ::VMEConfig::ConfigBase
        variable ModuleIndex
        variable ModuleName

        set ConfigFile [file join $ConfigBase [set ModuleName]#[set ModuleIndex].PLLtxt]

        set z [catch {open $ConfigFile r} file]
        if {$z != 0} {
            Alert "Unable to access $ConfigFile"
            InsertLog "[set file]"
            return
        }

        set Config {}
        set c 0
        foreach item [split [read $file] \n] {
            incr c
            if {$c <= 4} {continue}                    ;#  ignore file header
            if {[string length $item] != 20} {break}   ;#  ignore stuff at end
            set offset 0x[string range $item 1 2]
            set vl 0x[string range $item 17 18]
            lappend Config [list $offset $vl]
        }

        close $file

        return $Config
    }

variable V1751DPP_FOP_PAGE_PROG_TH_BUF1  0x0082              ;# V1751DPP flash page write
variable V1751DPP_FOP_PAGE_READ          0x00D2              ;# V1751DPP flash page read
variable V1751DPP_PLL_FLASH_PAGE         1
variable V1751DPP_FLASH_PAGE_SIZE        264

     proc PLLWriteFlash {} {
        variable ModuleIndex
        variable ModuleName
        variable V1751DPP_FOP_PAGE_PROG_TH_BUF1  
        variable V1751DPP_FOP_PAGE_READ
        variable V1751DPP_PLL_FLASH_PAGE 
        variable V1751DPP_FLASH_PAGE_SIZE

        set PLLData [GetPLLConfig]
        set flashaddr [expr $V1751DPP_PLL_FLASH_PAGE << 9]

        WriteRegister FlashEnable 0     ;#  enable flash

        WriteRegister FlashData  $V1751DPP_FOP_PAGE_PROG_TH_BUF1    ;# write OpCode

        WriteRegister FlashData [expr ($flashaddr >> 16) & 255]  ;# write address
        WriteRegister FlashData [expr ($flashaddr >> 8) & 255]
        WriteRegister FlashData [expr $flashaddr & 255]

        for {set i 0; set k 0} {$i < $V1751DPP_FLASH_PAGE_SIZE} {incr k} {   ;# write flash page

            if {$k < [llength $PLLData]} {
                 set Offset [first [lindex $PLLData $k]]
                 set Data [second [lindex $PLLData $k]]

            } else {
                 set Offset 255
                 set Data 255
            }
             
             WriteRegister FlashData $Offset
             WriteRegister FlashData $Data

             incr i 2
        }

        WriteRegister FlashEnable 1     ;#  disable flash

        WriteRegister ConfigReload 0    ;#  reload 

        Alert "PLL Flash Written for module [set ModuleName]#[set ModuleIndex]"

        after 250    ;#  allow time for reload 
     }

 
     proc PLLReadFlash {} {
        variable ModuleIndex
        variable ModuleName
        variable V1751DPP_FOP_PAGE_PROG_TH_BUF1  
        variable V1751DPP_FOP_PAGE_READ
        variable V1751DPP_PLL_FLASH_PAGE 
        variable V1751DPP_FLASH_PAGE_SIZE
        variable OutputClockEnable
        variable ClockDelayEnable
        variable ClockDelay

        set PLLData {}
        set flashaddr [expr $V1751DPP_PLL_FLASH_PAGE << 9]

        WriteRegister FlashEnable 0     ;#  enable flash

        WriteRegister FlashData  $V1751DPP_FOP_PAGE_READ    ;# write OpCode

        WriteRegister FlashData [expr ($flashaddr >> 16) & 255]  ;# write address
        WriteRegister FlashData [expr ($flashaddr >> 8) & 255]
        WriteRegister FlashData [expr $flashaddr & 255]

        for {set i 0; set k 0} {$i < $V1751DPP_FLASH_PAGE_SIZE} {incr k} {   ;# read flash page
             
             set Offset [format "0x%02x" [ReadRegister FlashData]]
             set Data [format "0x%02x" [ReadRegister FlashData]]

             if {$Offset == 0xff} {continue}

             if {$Offset == 0x38} {set ClockDelayEnable [expr (($Data & 1) + 1) & 1]}
             if {$Offset == 0x3a} {set ClockDelay [expr ($Data >> 1) & 31]}
             if {$Offset == 0x42} {set OutputClockEnable [expr (($Data & 1) + 1) & 1]}

             incr i 2

             lappend PLLData [list $Offset $Data]

             if {$Offset == 0x5a} {break}
        }

        WriteRegister FlashEnable 1     ;#  disable flash

        return $PLLData
     }

     proc PLLUpdateFlash {} {
        variable ModuleIndex
        variable ModuleName
        variable V1751DPP_FOP_PAGE_PROG_TH_BUF1  
        variable V1751DPP_FOP_PAGE_READ
        variable V1751DPP_PLL_FLASH_PAGE 
        variable V1751DPP_FLASH_PAGE_SIZE
        variable OutputClockEnable
        variable ClockDelayEnable
        variable ClockDelay

        set r38 $ClockDelayEnable          ;# save new values
        set r3a $ClockDelay
        set r42 $OutputClockEnable

        set PLLData [PLLReadFlash]
        set flashaddr [expr $V1751DPP_PLL_FLASH_PAGE << 9]

        WriteRegister FlashEnable 0     ;#  enable flash

        WriteRegister FlashData  $V1751DPP_FOP_PAGE_PROG_TH_BUF1    ;# write OpCode

        WriteRegister FlashData [expr ($flashaddr >> 16) & 255]  ;# write address
        WriteRegister FlashData [expr ($flashaddr >> 8) & 255]
        WriteRegister FlashData [expr $flashaddr & 255]

        for {set i 0; set k 0} {$i < $V1751DPP_FLASH_PAGE_SIZE} {incr k} {   ;# write flash page

            if {$k < [llength $PLLData]} {
                 set Offset [first [lindex $PLLData $k]]
                 set Data [second [lindex $PLLData $k]]

                 if {$Offset == 0x38} {     ;# delay bypass 
                     set Data [expr ($r38 + 1) & 1]
                 }
                 if {$Offset == 0x3a} {     ;# delay fine adjust
                     set Data [expr $r3a << 1]
                 }
                 if {$Offset == 0x42} {     ;# output
                     set op [expr ($r42 + 1) & 1]
                     set Data [expr ($Data & 0xfe) + $op]
                 }
                 if {$Offset == 0x55} {     ;# delay bypass
                     set op [expr ($r42 + 1) & 1]
                     set Data [expr ($Data & 0x7e) + ($op << 7)]
                 }

            } else {
                 set Offset 255
                 set Data 255
            }
             
             WriteRegister FlashData $Offset
             WriteRegister FlashData $Data

             incr i 2
        }

        WriteRegister FlashEnable 1     ;#  disable flash

        WriteRegister ConfigReload 0    ;#  reload 

        Alert "PLL Flash Written for module [set ModuleName]#[set ModuleIndex]"
        after 250    ;#  allow time for reload 
        Alert "Now restore settings"
     }

     proc PLLReloadFlash {} {

       WriteRegister ConfigReload 0    ;#  reload 

        after 250    ;#  allow time for reload 
        Alert "Now restore settings"
     }

